from ctypes import *
from enum import Enum, IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.common import *


class Fe_Caps(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    FE_IS_STUPID = 0

    (FE_CAN_INVERSION_AUTO, FE_CAN_FEC_1_2, FE_CAN_FEC_2_3, FE_CAN_FEC_3_4,
     FE_CAN_FEC_4_5, FE_CAN_FEC_5_6, FE_CAN_FEC_6_7, FE_CAN_FEC_7_8,
     FE_CAN_FEC_8_9, FE_CAN_FEC_AUTO, FE_CAN_QPSK, FE_CAN_QAM_16,
     FE_CAN_QAM_32, FE_CAN_QAM_64, FE_CAN_QAM_128, FE_CAN_QAM_256,
     FE_CAN_QAM_AUTO, FE_CAN_TRANSMISSION_MODE_AUTO, FE_CAN_BAUTO,
     FE_CAN_GUARD_INTERVAL_AUTO, FE_CAN_HIERARCHY_AUTO, FE_CAN_8VSB,
     FE_CAN_16VSB, FE_HAS_EXTENDED_CAPS) = [1 << x for x in range(24)]

    (FE_CAN_MULTISTREAM, FE_CAN_TURBO_FEC, FE_CAN_2G_MODULAT, FE_NEEDS_BENDING,
     FE_CAN_RECOVER, FE_CAN_MUTE_TS) = [1 << x for x in range(26, 32)]


class Fe_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (FE_QPSK, FE_QAM, FE_OFDM, FE_ATSC) = range(4)


class Dvb_Frontend_Info(Structure):
    _fields_ = [('name', c_char * 128), ('type', c_int),
                ('frequency_min', c_uint32), ('frequency_max', c_uint32),
                ('frequency_stepsize', c_uint32),
                ('frequency_tolerance', c_uint32),
                ('symbol_rate_min', c_uint32), ('symbol_rate_max', c_uint32),
                ('symbol_rate_tolerance', c_uint32),
                ('notifier_delay', c_uint32), ('caps', c_int)]


class Dvb_Diseqc_Master_Cmd(Structure):
    _fields_ = [('msg', c_uint8 * 6), ('msg_len', c_uint8)]


class Dvb_Diseqc_Slave_Reply(Structure):
    _fields_ = [('msg', c_uint8 * 4), ('msg_len', c_uint8),
                ('timeout', c_uint)]


class Fe_Sec_Voltage(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (S13, S18, SOFF) = range(3)


class Fe_Sec_Tone_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ON, OFF) = range(2)


class Fe_Sec_Mini_Cmd(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (A, B) = range(2)


class Fe_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)
        FE_NONE = 0x00,

        (FE_HAS_SIGNAL, FE_HAS_CARRIER, FE_HAS_VITERBI, FE_HAS_SYNC,
         FE_HAS_LOCK, FE_TIMEDOUT, FE_REINIT) = [1 << x for x in range(7)]


class Fe_Spectral_Inversion(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OFF, ON, AUTO) = range(3)


class Fe_Code_Rate(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    NONE = 0
    (C1_2, C2_3, C3_4, C4_5, C5_6, C6_7, C7_8, C8_9, CAUTO, C3_5, C9_10,
     C2_5) = range(12)


class Fe_Modulation(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)
        (QPSK, QAM_16, QAM_32, QAM_64, QAM_128, QAM_256, QAM_AUTO, VSB_8,
         VSB_16, PSK_8, APSK_16, APSK_32, DQPSK, QAM_4_NR) = range(14)


class Fe_Transmit_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (T2K, T8K, AUTO, T4K, T1K, T16K, T32K, C1, C3780) = range(9)


class Fe_Guard_Interval(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (G1_32, G1_16, G1_8, G1_4, AUTO, G1_128, G19_128, G19_256, PN420, PN595,
     PN945) = range(11)


class Fe_Hierarchy(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NONE, H1, H2, H4, AUTO) = range(5)


class Fe_Interleaving(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NONE, AUTO, I240, I720) = range(4)


class Dtv_(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (UNDEFINED, TUNE, CLEAR, FREQUENCY, MODULATION, BHZ, INVERSION,
     DISEQC_MASTER, SYMBOL_RATE, INNER_FEC, VOLTAGE, TONE, PILOT, ROLLOFF,
     DISEQC_SLAVE_REPLY, FE_CAPABILITY_COUNT, FE_CAPABILITY, DELIVERY_SYSTEM,
     ISDBT_PARTIAL_RECEPTION, ISDBT_SOUND_BROADCASTING, ISDBT_SB_SUBCHANNEL_ID,
     ISDBT_SB_SEGMENT_IDX, ISDBT_SB_SEGMENT_COUNT, ISDBT_LAYERA_FEC,
     ISDBT_LAYERA_MODULATION, ISDBT_LAYERA_SEGMENT_COUNT,
     ISDBT_LAYERA_TIME_INTERLEAVING, ISDBT_LAYERB_FEC, ISDBT_LAYERB_MODULATION,
     ISDBT_LAYERB_SEGMENT_COUNT, ISDBT_LAYERB_TIME_INTERLEAVING,
     ISDBT_LAYERC_FEC, ISDBT_LAYERC_MODULATION, ISDBT_LAYERC_SEGMENT_COUNT,
     ISDBT_LAYERC_TIME_INTERLEAVING, API_VERSION, CODE_RATE_HP, CODE_RATE_LP,
     GUARD_INTERVAL, TRANSMISSION_MODE, HIERARCHY, ISDBT_LAYER_ENABLED,
     STREAM_ID, DVBT2_PLP_ID_LEGACY, ENUM_DELSYS, ATSCMH_FIC_VER,
     ATSCMH_PARADE_ID, ATSCMH_NOG, ATSCMH_TNOG, ATSCMH_SGN, ATSCMH_PRC,
     ATSCMH_RS_FRAME_MODE, ATSCMH_RS_FRAME_ENSEMBLE, ATSCMH_RS_CODE_MODE_PRI,
     ATSCMH_RS_CODE_MODE_SEC, ATSCMH_SCCC_BLOCK_MODE, ATSCMH_SCCC_CODE_MODE_A,
     ATSCMH_SCCC_CODE_MODE_B, ATSCMH_SCCC_CODE_MODE_C, ATSCMH_SCCC_CODE_MODE_D,
     INTERLEAVING, LNA, STAT_SIGNAL_STRENGTH, STAT_CNR,
     STAT_PRE_ERROR_BIT_COUNT, STAT_PRE_TOTAL_BIT_COUNT,
     STAT_POST_ERROR_BIT_COUNT, STAT_POST_TOTAL_BIT_COUNT,
     STAT_ERROR_BLOCK_COUNT, STAT_TOTAL_BLOCK_COUNT,
     SCRAMBLING_SEQUENCE_INDEX) = range(71)

    ISDBS_TS_ID_LEGACY = STREAM_ID
    MAX_COMMAND = SCRAMBLING_SEQUENCE_INDEX


class Fe_Pilot(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ON, OFF, AUTO) = range(3)


class Fe_Rolloff(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (R35, R20, R25, AUTO) = range(4)


class Fe_Delivery_System(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SYS_UNDEFINED, SYS_DVBC_ANNEX_A, SYS_DVBC_ANNEX_B, SYS_DVBT, SYS_DSS,
     SYS_DVBS, SYS_DVBS2, SYS_DVBH, SYS_ISDBT, SYS_ISDBS, SYS_ISDBC, SYS_ATSC,
     SYS_ATSCMH, SYS_DTMB, SYS_CMMB, SYS_DAB, SYS_DVBT2, SYS_TURBO,
     SYS_DVBC_ANNEX_C) = range(19)
    SYS_DVBC_ANNEX_AC = SYS_DVBC_ANNEX_A
    SYS_DMBTH = SYS_DTMB


class Atscmh_Sccc_Block_mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SEP, COMB, RES) = range(3)


class Atscmh_Sccc_Code_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (HLF, QTR, RES) = range(3)


class Atscmh_Rs_Frame_Ensemble(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PRI, SEC) = range(2)


class Atscmh_Rs_Frame_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PRI_ONLY, PRI_SEC, RES) = range(3)


class Atscmh_Rs_Code_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (R211_187, R223_187, R235_187, RES) = range(4)


class Dvb_Sys(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    NO_STREAM_ID_FILTER = ~c_uint32(0).value
    LNA_AUTO = ~c_uint32(0).value


def v4l2_ctrl_id2which(id):
    c = 0x0fff0000
    return id & c_ulong(c).value


class Fecap_Scale_Params(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    NOT_AVAILABLE = 0,
    (DECIBEL, RELATIVE, COUNTER) = range(3)


class Dtv_Stats(Structure):
    class _u(Union):
        _fields_ = [('uvalue', c_uint64), ('svalue', c_int64)]

    _fields_ = [('scale', c_uint8), ('_u', _u)]
    _pack_ = True


class Dtv_Statistics(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    MAX = 4


class Dtv_Fe_Stats(Structure):
    _fields_ = [('len', c_uint8), ('stat', Dtv_Stats * Dtv_Statistics.MAX)]
    _pack_ = True


class Dtv_Property(Structure):
    class _u(Union):
        class _s(Structure):
            _fields_ = [('data', c_uint8 * 32), ('len', c_uint32),
                        ('reserved1', c_uint32 * 3), ('reserved2', c_void_p)]

        _fields_ = [('data', c_uint32), ('st', Dtv_Fe_Stats), ('buffer', _s)]

    _fields_ = [('cmd', c_uint32), ('reserved', c_uint32 * 3), ('u', _u),
                ('result', c_int)]
    _pack_ = True


class Dtv_Msgs(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    MAX = 64


class Dtv_Properties(Structure):
    _fields_ = [('num', c_uint32), ('props', POINTER(Dtv_Property))]


class Tune_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ONESHOT = 0x01


class Fe_Bandwidth(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (B8_MHZ, B7_MHZ, B6_MHZ, AUTO, B5_MHZ, B10_MHZ, B1_712_MHZ) = range(7)


Fe_Sec_Voltage_T = Fe_Sec_Voltage
Fe_Caps_T = Fe_Caps
Fe_Type_T = Fe_Type
Fe_Sec_Tone_Mode_T = Fe_Sec_Tone_Mode
Fe_Sec_Mini_Cmd_T = Fe_Sec_Mini_Cmd
Fe_Status_T = Fe_Status
Fe_Spectral_Inversion_T = Fe_Spectral_Inversion
Fe_Code_Rate_T = Fe_Code_Rate
Fe_Modulation_T = Fe_Modulation
Fe_Transmit_Mode_T = Fe_Transmit_Mode
Fe_Bandwidth_T = Fe_Bandwidth
Fe_Guard_Interval_T = Fe_Guard_Interval
Fe_Hierarchy_T = Fe_Hierarchy
Fe_Pilot_T = Fe_Pilot
Fe_Rolloff_T = Fe_Rolloff
Fe_Delivery_System_T = Fe_Delivery_System


class Dvb_Qpsk_Parameters(Structure):
    _fields_ = [
        ('symbol_rate', c_uint32),
        ('fec_inner', c_uint32),
    ]


class Dvb_Qam_Parameters(Structure):
    _fields_ = [('symbol_rate', c_uint32), ('fec_inner', c_uint32),
                ('modulation', c_uint32)]


class Dvb_Vsb_Parameters(Structure):
    _fields_ = [
        ('modulation', c_uint32),
    ]


class Dvb_Ofdm_Parameters(Structure):
    _fields_ = [('bandwidth', c_uint32), ('code_rate_HP', c_uint32),
                ('code_rate_LP', c_uint32), ('constellation', c_uint32),
                ('transmission_mode', c_uint32), ('guard_interval', c_uint32),
                ('hierarchy_information', c_uint32)]


class Dvb_Frontend_Parameters(Structure):
    class _u(Union):
        _fields_ = [('qpsk', Dvb_Qpsk_Parameters), ('qam', Dvb_Qam_Parameters),
                    ('ofdm', Dvb_Ofdm_Parameters), ('vsb', Dvb_Vsb_Parameters)]

    _fields_ = [('frequency', c_uint32), ('inversion', c_uint32), ('u', _u)]


class Dvb_Frontend_Event(Structure):
    _fields_ = [('status', c_uint32), ('parameters', Dvb_Frontend_Parameters)]


class Fe(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    GET_INFO = ior('o', 61, Dvb_Frontend_Info)
    DISEQC_RESET_OVERLOAD = io('o', 62)
    DISEQC_SEND_MASTER_CMD = iow('o', 63, Dvb_Diseqc_Master_Cmd)
    DISEQC_RECV_SLAVE_REPLY = ior('o', 64, Dvb_Diseqc_Slave_Reply)
    DISEQC_SEND_BURST = io('o', 65)
    SET_TONE = io('o', 66)
    SET_VOLTAGE = io('o', 67)
    ENABLE_HIGH_LNB_VOLTAGE = io('o', 68)
    READ_STATUS = ior('o', 69, c_uint32)
    READ_BER = ior('o', 70, c_uint32)
    READ_SIGNAL_STRENGTH = ior('o', 71, c_uint16)
    READ_SNR = ior('o', 72, c_uint16)
    READ_UNCORRECTED_BLOCKS = ior('o', 73, c_uint32)
    SET_FRONTEND_TUNE_MODE = io('o', 81)
    GET_EVENT = ior('o', 78, Dvb_Frontend_Event)
    DISHNETWORK_SEND_LEGACY_CMD = io('o', 80)
    SET_PROPERTY = iow('o', 82, Dtv_Properties)
    GET_PROPERTY = ior('o', 83, Dtv_Properties)
    SET_FRONTEND = iow('o', 76, Dvb_Frontend_Parameters)
    GET_FRONTEND = ior('o', 77, Dvb_Frontend_Parameters)



from ctypes import *
from enum import Enum, IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.common import *


class Cec_Max(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    MSG_SIZE = 16
    LOG_ADDRS = 4


class Cec_Msg(Structure):
    _fields_ = [('tx_ts', c_uint64), ('rx_ts', c_uint64), ('len', c_uint32),
                ('timeout', c_uint32), ('sequence', c_uint32),
                ('flags', c_uint32), ('msg', c_uint8 * Cec_Max.MSG_SIZE),
                ('reply', c_uint8), ('rx_status', c_uint8),
                ('tx_status', c_uint8), ('tx_arb_lost_cnt', c_uint8),
                ('tx_nack_cnt', c_uint8), ('tx_low_drive_cnt', c_uint8),
                ('tx_error_cnt', c_uint8)]


def cec_msg_initiator(msg: POINTER(Cec_Msg)) -> c_uint8:
    return msg.contents.tx_ts >> 4


def cec_msg_destination(msg: POINTER(Cec_Msg)) -> c_uint8:
    return msg.contents.tx_ts & 0xf


def cec_msg_opcode(msg: POINTER(Cec_Msg)) -> c_int:
    return (-1, msg.contents.rx_ts)[msg.contents.len > 1]


def cec_msg_is_broadcast(msg: POINTER(Cec_Msg)) -> c_int:
    return (msg.contents.tx_ts & 0xf) == 0xf


def cec_msg_init(
        msg: POINTER(Cec_Msg), initiator: c_uint8, destination: c_uint8):
    memset(msg, 0, sizeof(pointer(msg)))
    msg.contents.tx_ts = (initiator << 4) | destination
    msg.contents.len = 1


def cec_msg_set_reply_to(msg: POINTER(Cec_Msg), orig: POINTER(Cec_Msg)):
    msg.contents.tx_ts = (
        cec_msg_destination(orig) << 4) | cec_msg_initiator(orig)
    msg.contents.reply = msg.timeout = 0


class Cec_Msg_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (REPLY_TO_FOLLOWERS, RAW) = [1 << x for x in range(2)]


class Cec_Tx_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OK, ARB_LOST, NACK, LOW_DRIVE, ERROR, MAX_RETRIES, ABORTED,
     TIMEOUT) = [1 << x for x in range(8)]


class Cec_Rx_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OK, TIMEOUT, FEATURE_ABORT, ABORTED) = [1 << x for x in range(4)]


def cec_msg_status_is_ok(msg: POINTER(Cec_Msg)) -> c_int:
    if (msg.tx_status and not (msg.tx_status & Cec_Tx_Status.OK)):
        return 0
    if (msg.rx_status and not (msg.rx_status & Cec_Rx_Status.OK)):
        return 0
    if (not msg.tx_status and not msg.rx_status):
        return 0
    return not (msg.rx_status & Cec_Rx_Status.FEATURE_ABORT)


class Cec_Phys_Addr(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    INVALID = 0xffff


class Cec_Log_Addr(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    INVALID = 0xff
    (TV, RECORD_1, RECORD_2, TUNER_1, PLAYBACK_1, AUDIOSYSTEM, TUNER_2,
     TUNER_3, PLAYBACK_2, RECORD_3, TUNER_4, PLAYBACK_3, BACKUP_1, BACKUP_2,
     SPECIFIC, UNREGISTERED) = range(16)
    BROADCAST = 15


class Cec_Log_Addr_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (TV, RECORD, TUNER, PLAYBACK, AUDIOSYSTEM, SPECIFIC,
     UNREGISTERED) = range(7)


class Cec_Log_Addr_Mask(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    TV = (1 << Cec_Log_Addr.TV)
    RECORD = ((1 << Cec_Log_Addr.RECORD_1) | (1 << Cec_Log_Addr.RECORD_2) |
              (1 << Cec_Log_Addr.RECORD_3))
    TUNER = ((1 << Cec_Log_Addr.TUNER_1) | (1 << Cec_Log_Addr.TUNER_2) |
             (1 << Cec_Log_Addr.TUNER_3) | (1 << Cec_Log_Addr.TUNER_4))
    PLAYBACK = ((1 << Cec_Log_Addr.PLAYBACK_1) | (1 << Cec_Log_Addr.PLAYBACK_2)
                | (1 << Cec_Log_Addr.PLAYBACK_3))
    AUDIOSYSTEM = (1 << Cec_Log_Addr.AUDIOSYSTEM)
    BACKUP = ((1 << Cec_Log_Addr.BACKUP_1) | (1 << Cec_Log_Addr.BACKUP_2))
    SPECIFIC = (1 << Cec_Log_Addr.SPECIFIC)
    UNREGISTERED = (1 << Cec_Log_Addr.UNREGISTERED)


def cec_has_tv(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.TV


def cec_has_record(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.RECORD


def cec_has_tuner(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.TUNER


def cec_has_playback(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.PLAYBACK


def cec_has_audiosystem(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.AUDIOSYSTEM


def cec_has_backup(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.BACKUP


def cec_has_specific(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.SPECIFIC


def cec_is_unregistered(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask & Cec_Log_Addr_Mask.UNREGISTERED


def cec_is_unconfigured(log_addr_mask: c_uint16) -> c_int:
    return log_addr_mask == 0


class Cec_Vendor(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ID_NONE = 0xffffffff


class Cec_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NO_INITIATOR, INITIATOR, EXCL_INITIATOR) = [x << 0 for x in range(3)]

    INITIATOR_MSK = 0x0f

    (NO_FOLLOWER, FOLLOWER, EXCL_FOLLOWER,
     EXCL_FOLLOWER_PASSTHRU) = [x << 4 for x in range(4)]

    (MONITOR_PIN, MONITOR, MONITOR_ALL) = [x << 4 for x in range(13, 16)]

    FOLLOWER_MSK = 0xf0


class Cec_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PHYS_ADDR, LOG_ADDRS, TRANSMIT, PASSTHROUGH, RC, MONITOR_ALL, NEEDS_HPD,
     MONITOR_PIN) = [1 << x for x in range(8)]


class Cec_Caps(Structure):
    _fields_ = [
        ('driver', c_char * 32),
        ('name', c_char * 32),
        ('available_log_addrs', c_uint32),
        ('capabilities', c_uint32),
        ('version', c_uint32),
    ]


class Cec_Log_Addrs(Structure):
    _fields_ = [('log_addr', c_uint8 * Cec_Max.LOG_ADDRS),
                ('log_addr_mask', c_uint16), ('cec_version', c_uint8),
                ('num_log_addrs', c_uint8), ('vendor_id', c_uint32),
                ('flags', c_uint32), ('osd_name', c_char * 15),
                ('primary_device_type', c_uint8 * Cec_Max.LOG_ADDRS),
                ('log_addr_type', c_uint8 * Cec_Max.LOG_ADDRS),
                ('all_device_types', c_uint8 * Cec_Max.LOG_ADDRS),
                ('features', c_uint8 * Cec_Max.LOG_ADDRS * 12)]


class Cec_Log_Addrs_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ALLOW_UNREG_FALLBACK, ALLOW_RC_PASSTHRU,
     CDC_ONLY) = [1 << x for x in range(3)]


class Cec_Event_Enum(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (STATE_CHANGE, LOST_MSGS, PIN_CEC_LOW, PIN_CEC_HIGH, PIN_HPD_LOW,
     PIN_HPD_HIGH, PIN_5V_LOW, PIN_5V_HIGH) = range(1, 9)


class Cec_Event_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (INITIAL_STATE, DROPPED_EVENTS) = [1 << x for x in range(2)]


class Cec_Event_State_Change(Structure):
    _fields_ = [('phys_addr', c_uint16), ('log_addr_mask', c_uint16)]


class Cec_Event_Lost_Msgs(Structure):
    _fields_ = [('lost_msgs', c_uint32)]


class Cec_Event(Structure):
    class _u(Union):
        _field_ = [('state_change', Cec_Event_State_Change),
                   ('lost_msgs', Cec_Event_Lost_Msgs), ('raw', c_uint32)]

    _fields_ = [('ts', c_uint64), ('event', c_uint32), ('flags', c_uint32),
                ('_u', _u)]


class Cec(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ADAP_G_CAPS = iowr('a', 0, Cec_Caps)
    ADAP_G_PHYS_ADDR = ior('a', 1, c_uint16)
    ADAP_S_PHYS_ADDR = iow('a', 2, c_uint16)
    ADAP_G_LOG_ADDRS = ior('a', 3, Cec_Log_Addrs)
    ADAP_S_LOG_ADDRS = iowr('a', 4, Cec_Log_Addrs)
    TRANSMIT = iowr('a', 5, Cec_Msg)
    RECEIVE = iowr('a', 6, Cec_Msg)
    DQEVENT = iowr('a', 7, Cec_Event)
    G_MODE = ior('a', 8, c_uint32)
    S_MODE = iow('a', 9, c_uint32)


class Cec_Msg_Enum(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ACTIVE_SOURCE = 0x82
    IMAGE_VIEW_ON = 0x04
    TEXT_VIEW_ON = 0x0d
    INACTIVE_SOURCE = 0x9d
    REQUEST_ACTIVE_SOURCE = 0x85
    ROUTING_CHANGE = 0x80
    ROUTING_INFORMATION = 0x81
    SET_STREAM_PATH = 0x86
    STANDBY = 0x36
    RECORD_OFF = 0x0b
    RECORD_ON = 0x09
    RECORD_STATUS = 0x0a
    RECORD_TV_SCREEN = 0x0f
    CLEAR_ANALOGUE_TIMER = 0x33
    CLEAR_DIGITAL_TIMER = 0x99
    CLEAR_EXT_TIMER = 0xa1
    SET_ANALOGUE_TIMER = 0x34
    SET_DIGITAL_TIMER = 0x97
    SET_EXT_TIMER = 0xa2
    SET_TIMER_PROGRAM_TITLE = 0x67
    TIMER_CLEARED_STATUS = 0x43
    TIMER_STATUS = 0x35
    CEC_VERSION = 0x9e
    SET_MENU_LANGUAGE = 0x32
    REPORT_FEATURES = 0xa6
    GET_CEC_VERSION = 0x9f
    GIVE_PHYSICAL_ADDR = 0x83
    GET_MENU_LANGUAGE = 0x91
    REPORT_PHYSICAL_ADDR = 0x84
    GIVE_FEATURES = 0xa5
    DECK_CONTROL = 0x42
    DECK_STATUS = 0x1b
    GIVE_DECK_STATUS = 0x1a
    GIVE_TUNER_DEVICE_STATUS = 0x08
    SELECT_ANALOGUE_SERVICE = 0x92
    SELECT_DIGITAL_SERVICE = 0x93
    TUNER_DEVICE_STATUS = 0x07
    PLAY = 0x41
    TUNER_STEP_DECREMENT = 0x06
    TUNER_STEP_INCREMENT = 0x05
    DEVICE_VENDOR_ID = 0x87
    GIVE_DEVICE_VENDOR_ID = 0x8c
    VENDOR_COMMAND = 0x89
    VENDOR_COMMAND_WITH_ID = 0xa0
    VENDOR_REMOTE_BUTTON_DOWN = 0x8a
    VENDOR_REMOTE_BUTTON_UP = 0x8b
    SET_OSD_STRING = 0x64
    GIVE_OSD_NAME = 0x46
    SET_OSD_NAME = 0x47
    MENU_REQUEST = 0x8d
    MENU_STATUS = 0x8e
    USER_CONTROL_PRESSED = 0x44
    USER_CONTROL_RELEASED = 0x45
    GIVE_DEVICE_POWER_STATUS = 0x8f
    REPORT_POWER_STATUS = 0x90
    ABORT = 0xff
    GIVE_AUDIO_STATUS = 0x71
    GIVE_SYSTEM_AUDIO_MODE_STATUS = 0x7d
    REPORT_AUDIO_STATUS = 0x7a
    REPORT_SHORT_AUDIO_DESCRIPTOR = 0xa3
    REQUEST_SHORT_AUDIO_DESCRIPTOR = 0xa4
    SET_SYSTEM_AUDIO_MODE = 0x72
    SYSTEM_AUDIO_MODE_REQUEST = 0x70
    SYSTEM_AUDIO_MODE_STATUS = 0x7e
    SET_AUDIO_RATE = 0x9a
    FEATURE_ABORT = 0x00
    INITIATE_ARC = 0xc0
    REPORT_ARC_INITIATED = 0xc1
    REPORT_ARC_TERMINATED = 0xc2
    REQUEST_ARC_INITIATION = 0xc3
    REQUEST_ARC_TERMINATION = 0xc4
    TERMINATE_ARC = 0xc5
    REQUEST_CURRENT_LATENCY = 0xa7
    REPORT_CURRENT_LATENCY = 0xa8
    CDC_MESSAGE = 0xf8
    CDC_HEC_INQUIRE_STATE = 0x00
    CDC_HEC_REPORT_STATE = 0x01
    CDC_HEC_SET_STATE_ADJACENT = 0x02
    CDC_HEC_SET_STATE = 0x03
    CDC_HEC_REQUEST_DEACTIVATION = 0x04
    CDC_HEC_NOTIFY_ALIVE = 0x05
    CDC_HEC_DISCOVER = 0x06
    CDC_HPD_SET_STATE = 0x10
    CDC_HPD_REPORT_STATE = 0x11


class Cec_Op_Record_Src(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OWN, DIGITAL, ANALOG, EXT_PLUG, EXT_PHYS_ADDR) = range(1, 6)


class Cec_Op_Service_ID_Method(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (BY_DIG_ID, BY_CHANNEL) = range(2)


class Cec_Op_Dig_Service_Bcast_System(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ARIB_GEN, ATSC_GEN, DVB_GEN) = range(3)

    (ARIB_BS, ARIB_CS, ARIB_T) = range(8, 11)

    (ATSC_CABLE, ATSC_SAT, ATSC_T) = range(16, 19)

    (DVB_C, DVB_S, DVB_S2, DVB_T) = range(24, 28)


class Cec_Op_Ana_Bcast_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (CABLE, SATELLITE, TERRESTRIAL) = range(3)


class Cec_Op_Bcast_System(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PAL_BG, SECAM_LQ, PAL_M, NTSC_M, PAL_I, SECAM_DK, SECAM_BG, SECAM_L,
     PAL_DK) = range(0x9)

    OTHER = 0x1f


class Cec_Op_Channel_Number(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (FMT_1_PART, FMT_2_PART) = range(1, 3)


class Cec_Op_Record_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (CUR_SRC, DIG_SERVICE, ANA_SERVICE, EXT_INPUT, NO_DIG_SERVICE,
     NO_ANA_SERVICE, NO_SERVICE) = range(1, 8)

    (INVALID_EXT_PLUG, INVALID_EXT_PHYS_ADDR, UNSUP_CA, NO_CA_ENTITLEMENTS,
     CANT_COPY_SRC, NO_MORE_COPIES) = range(9, 15)

    (NO_MEDIA, PLAYING, ALREADY_RECORDING, MEDIA_PROT, NO_SIGNAL,
     MEDIA_PROBLEM, NO_SPACE, PARENTAL_LOCK) = range(16, 24)

    (TERMINATED_OK, ALREADY_TERM) = range(26, 28)
    OTHER = 0x1f


class Cec_Op_Rec_Seq(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY,
     SATERDAY) = [1 << x for x in range(7)]

    ONCE_ONLY = 0x00


class Cec_Op_Ext_Src(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PLUG, PHYS_ADDR) = range(4, 6)


class Cec_Op_Timer_Clr_Stat(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (RECORDING, NO_MATCHING, NO_INFO) = range(3)

    CLEARED = 0x80


class Cec_Op_Timer_Overlap_Warning(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NO_OVERLAP, OVERLAP) = range(2)


class Cec_Op_Media_Info(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (UNPROT_MEDIA, PROT_MEDIA, NO_MEDIA) = range(3)


class Cec_Op_Prog_Ind(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NOT_PROGRAMMED, PROGRAMMED) = range(2)


class Cec_Op_Prog_Info(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ENOUGH_SPACE, NOT_ENOUGH_SPACE, NONE_AVAILABLE,
     MIGHT_NOT_BE_ENOUGH_SPACE) = range(8, 12)


class Cec_Op_Prog_Error(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NO_FREE_TIMER, DATE_OUT_OF_RANGE, REC_SEQ_ERROR, INV_EXT_PLUG,
     INV_EXT_PHYS_ADDR, CA_UNSUPP, INSUF_CA_ENTITLEMENTS, RESOLUTION_UNSUPP,
     PARENTAL_LOCK, CLOCK_FAILURE) = range(1, 11)

    DUPLICATE = 0x0e


class Cec_Op_Cec(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (VERSION_1_3A, VERSION_1_4, VERSION_2_0) = range(4, 7)


class Cec_Op_Prim_Devtype(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (TV, RECORD) = range(2)

    (TUNER, PLAYBACK, AUDIOSYSTEM, SWITCH, PROCESSOR) = range(3, 8)


class Cec_Op_All_Devtype(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SWITCH, AUDIOSYSTEM, PLAYBACK, TUNER, RECORD,
     TV) = [1 << x for x in range(2, 8)]


class Cec_Op_Feat(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    EXT = 0x80


class Cec_Op_Feat_Rc_Tv(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    PROFILE_NONE = 0x00

    (PROFILE_1, PROFILE_2, PROFILE_3, PROFILE_4) = range(2, 15, 4)


class Cec_Op_Feat_Rc_Src(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    HAS_DEV_ROOT_MENU = 0x50
    HAS_DEV_SETUP_MENU = 0x48
    HAS_CONTENTS_MENU = 0x44
    HAS_MEDIA_TOP_MENU = 0x42
    HAS_MEDIA_CONTEXT_MENU = 0x41


class Cec_Op_Feat_Dev(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SOURCE_HAS_ARC_RX, SINK_HAS_ARC_TX, HAS_SET_AUDIO_RATE, HAS_DECK_CONTROL,
     HAS_SET_OSD_STRING, HAS_RECORD_TV_SCREEN) = [1 << x for x in range(2, 8)]


class Cec_Op_Deck_Ctl_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SKIP_FWD, SKIP_REV, STOP, EJECT) = range(1, 5)


class Cec_Op_Deck_Info(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PLAY, RECORD, PLAY_REV, STILL, SLOW, SLOW_REV, FAST_FWD, FAST_REV,
     NO_MEDIA, STOP, SKIP_FWD, SKIP_REV, INDEX_SEARCH_FWD, INDEX_SEARCH_REV,
     OTHER) = range(17, 32)


class Cec_Op_Status_Req(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ON, OFF, ONCE) = range(1, 4)


class Cec_Op_Play_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PLAY_FAST_FWD_MIN, PLAY_FAST_FWD_MED, PLAY_FAST_FWD_MAX) = range(5, 8)

    (PLAY_FAST_REV_MIN, PLAY_FAST_REV_MED, PLAY_FAST_REV_MAX) = range(9, 12)

    (PLAY_SLOW_FWD_MIN, PLAY_SLOW_FWD_MED, PLAY_SLOW_FWD_MAX) = range(21, 24)

    (PLAY_SLOW_REV_MIN, PLAY_SLOW_REV_MED, PLAY_SLOW_REV_MAX) = range(25, 28)

    PLAY_REV = 0x20

    (PLAY_FWD, PLAY_STILL) = range(36, 38)


class Cec_Op_Rec_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NOT_USED, USED) = range(2)


class Cec_Op_Tuner_Display_Info(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DIGITAL, NONE, ANALOGUE) = range(3)


class Cec_Op_Disp_Ctl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    DEFAULT = 0x00
    (UNTIL_CLEARED, CLEAR) = [1 << x for x in range(6, 8)]


class Cec_Op_Menu_Request(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ACTIVATE, DEACTIVATE, QUERY) = range(3)


class Cec_Op_Menu_State(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ACTIVATED, DEACTIVATED) = range(2)


class Cec_Op_Ui_Bcast_Type_Toggle(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ALL, DIG_ANA) = range(2)


class Cec_Op_Ui_Bcast_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ANALOGUE, ANALOGUE_T, ANALOGUE_CABLE, ANALOGUE_SAT, DIGITAL, DIGITAL_T,
     DIGITAL_CABLE, DIGITAL_SAT, DIGITAL_COM_SAT, IP) = range(16, 161, 16)

    DIGITAL_COM_SAT2 = 0x91


class Cec_Op_Ui_Snd_Pres_Ctl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DUAL_MONO, KARAOKE) = range(16, 33, 16)

    (DOWNMIX, REVERB, EQUALIZER) = range(128, 161, 16)


class Cec_Op_Ui_Snd_Pres_Ctl_Bass(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (UP, NEUTRAL, DOWN) = range(177, 180)


class Cec_Op_Ui_Snd_Pres_Ctl_Treble(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (UP, NEUTRAL, DOWN) = range(193, 196)


class Cec_Op_Power_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ON, STANDBY, TO_ON, TO_STANDBY) = range(4)


class Cec_Op_Abort(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (UNRECOGNIZED_OP, INCORRECT_MODE, NO_SOURCE, INVALID_OP, REFUSED,
     UNDETERMINED) = range(6)


class Cec_Op_Aud_Mute_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OFF, ON) = range(2)


class Cec_Op_Sys_Aud_Status(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OFF, ON) = range(2)


class Cec_Op_Aud_Fmt_Id(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (CEA861, CEA861_CXT) = range(2)


class Cec_Op_Aud_Rate(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OFF, WIDE_STD, WIDE_FAST, WIDE_SLOW, NARROW_STD, NARROW_FAST,
     NARROW_SLOW) = range(7)


class Cec_Op_Low_Latency_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (OFF, ON) = range(2)


class Cec_Op_Aud_Out_Compensated(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NA, DELAY, NO_DELAY, PARTIAL_DELAY) = range(4)


class Cec_Op_Hec_Func_State(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NOT_SUPPORTED, INACTIVE, ACTIVE, ACTIVATION_FIELD) = range(4)


class Cec_Op_Host_Func_State(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NOT_SUPPORTED, INACTIVE, ACTIVE) = range(3)


class Cec_Op_Enc_Func_State_Ext_Con(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NOT_SUPPORTED, INACTIVE, ACTIVE) = range(3)


class Cec_Op_Cdc_Error_Code(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NONE, CAP_UNSUPPORTED, WRONG_STATE, OTHER) = range(4)


class Cec_Op_Hec_Support(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NO, YES) = range(2)


class Cec_Op_Hec_Activation(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ON, OFF) = range(2)


class Cec_Op_Hec_Set_State(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DEACTIVATE, ACTIVATE) = range(2)


class Cec_Op_Hpd_State(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (CP_EDID_DISABLE, CP_EDID_ENABLE, CP_EDID_DISABLE_ENABLE, EDID_DISABLE,
     EDID_ENABLE, EDID_DISABLE_ENABLE) = range(6)


class Cec_Op_Hpd_Error(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NONE, INITIATOR_NOT_CAPABLE, INITIATOR_WRONG_STATE, OTHER,
     NONE_NO_VIDEO) = range(5)


def cec_is_2nd_tv(las: POINTER(Cec_Log_Addrs)) -> c_int:
    return las.contents.num_log_addrs and \
        las.contents.log_addr[0] >= Cec_Log_Addr.SPECIFIC and \
        las.contents.primary_device_type[0] == Cec_Op_Prim_Devtype.TV


def cec_is_processor(las: POINTER(Cec_Log_Addrs)) -> c_int:
    return las.contents.num_log_addrs and \
           las.contents.log_addr[0] >= Cec_Log_Addr.BACKUP_1 and \
           las.contents.primary_device_type[0] == Cec_Op_Prim_Devtype.PROCESSOR


def cec_is_switch(las: POINTER(Cec_Log_Addrs)) -> c_int:
    return las.contents.num_log_addrs == 1 and \
           las.contents.log_addr[0] == Cec_Log_Addr.UNREGISTERED and \
           las.contents.primary_device_type[0] == Cec_Op_Prim_Devtype.SWITCH and \
           not(las.contents.flags & Cec_Log_Addrs_Fl.CDC_ONLY)


def cec_is_cdc_only(las: POINTER(Cec_Log_Addrs)) -> c_int:
    return las.contents.num_log_addrs == 1 and \
           las.contents.log_addr[0] == Cec_Log_Addr.UNREGISTERED and \
           las.contents.primary_device_type[0] == Cec_Op_Prim_Devtype.SWITCH and \
           (las.contents.flags & Cec_Log_Addrs_Fl.CDC_ONLY)



from ctypes import *
from enum import Enum, IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.videodev import *


class V4l2_Mbus_Framefmt(Structure):
    _fields_ = [('width', c_uint32), ('height', c_uint32), ('code', c_uint),
                ('field', c_uint32), ('colorspace', c_uint32),
                ('ycbcr_enc', c_uint16), ('quantization', c_uint16),
                ('xfer_func', c_uint16), ('reserved', c_uint16 * 11)]



from ctypes import *
from enum import IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.mediabus import *
from v4l2.videodev import *


class V4l2_Subdev_Format_Whence(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (TRY, ACTIVE) = range(2)


class V4l2_Subdev_Format(Structure):
    _fields_ = [('which', c_uint32), ('pad', c_uint32),
                ('format', V4l2_Mbus_Framefmt), ('reserved', c_uint32 * 8)]


class V4l2_Subdev_Crop(Structure):
    _fields_ = [('which', c_uint32), ('pad', c_uint32), ('rect', V4l2_Rect),
                ('reserved', c_uint32 * 8)]


class V4l2_Subdev_Mbus_Code_Enum(Structure):
    _fields_ = [('pad', c_uint32), ('index', c_uint32), ('code', c_uint32),
                ('which', c_uint32), ('reserved', c_uint32 * 8)]


class V4l2_Subdev_Frame_Size_Enum(Structure):
    _fields_ = [('index', c_uint32), ('pad', c_uint32), ('code', c_uint32),
                ('min_width', c_uint32), ('max_width', c_uint32),
                ('min_height', c_uint32), ('max_height', c_uint32),
                ('which', c_uint32), ('reserved', c_uint32 * 8)]


class V4l2_Subdev_Frame_Interval(Structure):
    _fields_ = [('pad', c_uint32), ('interval', V4l2_Fract),
                ('reserved', c_uint32 * 9)]


class V4l2_Subdev_Frame_Interval_Enum(Structure):
    _fields_ = [('index', c_uint32), ('pad', c_uint32), ('code', c_uint32),
                ('width', c_uint32), ('height', c_uint32),
                ('interval', V4l2_Fract), ('which', c_uint32),
                ('reserved', c_uint32 * 8)]


class V4l2_Subdev_Selection(Structure):
    _fields_ = [
        ('which', c_uint32),
        ('pad', c_uint32),
        ('target', c_uint32),
        ('flags', c_uint32),
        ('r', V4l2_Rect),
        ('reserved', c_uint32 * 8),
    ]


class V4l2_Subdev_Capability(Structure):
    _fields_ = [('version', c_uint32), ('capabilities', c_uint32),
                ('reserved', c_uint32 * 14)]


class V4l2_Subdev_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    RO_SUBDEV = 0x00000001


V4l2_Subdev_Edid = V4l2_Edid


class Vidioc_Subdev(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    QUERYCAP = ior('V', 0, V4l2_Subdev_Capability)
    G_FMT = iowr('V', 4, V4l2_Subdev_Format)
    S_FMT = iowr('V', 5, V4l2_Subdev_Format)
    G_FRAME_INTERVAL = iowr('V', 21, V4l2_Subdev_Frame_Interval)
    S_FRAME_INTERVAL = iowr('V', 22, V4l2_Subdev_Frame_Interval)
    ENUM_MBUS_CODE = iowr('V', 2, V4l2_Subdev_Mbus_Code_Enum)
    ENUM_FRAME_SIZE = iowr('V', 74, V4l2_Subdev_Frame_Size_Enum)
    ENUM_FRAME_INTERVAL = iowr('V', 75, V4l2_Subdev_Frame_Interval_Enum)
    G_CROP = iowr('V', 59, V4l2_Subdev_Crop)
    S_CROP = iowr('V', 60, V4l2_Subdev_Crop)
    G_SELECTION = iowr('V', 61, V4l2_Subdev_Selection)
    S_SELECTION = iowr('V', 62, V4l2_Subdev_Selection)
    G_STD = ior('V', 23, V4l2_Std_Id)
    S_STD = iow('V', 24, V4l2_Std_Id)
    ENUMSTD = iowr('V', 25, V4l2_Standard)
    G_EDID = iowr('V', 40, V4l2_Edid)
    S_EDID = iowr('V', 41, V4l2_Edid)
    QUERYSTD = ior('V', 63, V4l2_Std_Id)
    S_DV_TIMINGS = iowr('V', 87, V4l2_Dv_Timings)
    G_DV_TIMINGS = iowr('V', 88, V4l2_Dv_Timings)
    ENUM_DV_TIMINGS = iowr('V', 98, V4l2_Enum_Dv_Timings)
    QUERY_DV_TIMINGS = ior('V', 99, V4l2_Dv_Timings)
    DV_TIMINGS_CAP = iowr('V', 100, V4l2_Dv_Timings_Cap)



from ctypes import *
from enum import Enum, IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.common import *


class Media_Device_Info(Structure):
    _fields_ = [('driver', c_char * 16), ('model', c_char * 32),
                ('serial', c_char * 40), ('bus_info', c_char * 32),
                ('media_version', c_uint32), ('hw_revision', c_uint32),
                ('driver_version', c_uint32), ('reserved', c_uint32 * 31)]


class Media_Ent_F(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    BASE = 0x00000000
    OLD_BASE = 0x00010000
    OLD_SUBDEV_BASE = 0x00020000
    UNKNOWN = BASE
    V4L2_SUBDEV_UNKNOWN = OLD_SUBDEV_BASE
    DTV_DEMOD = (BASE + 0x00001)
    TS_DEMUX = (BASE + 0x00002)
    DTV_CA = (BASE + 0x00003)
    DTV_NET_DECAP = (BASE + 0x00004)
    IO_V4L = (OLD_BASE + 1)
    IO_DTV = (BASE + 0x01001)
    IO_VBI = (BASE + 0x01002)
    IO_SWRADIO = (BASE + 0x01003)
    CAM_SENSOR = (OLD_SUBDEV_BASE + 1)
    FLASH = (OLD_SUBDEV_BASE + 2)
    LENS = (OLD_SUBDEV_BASE + 3)
    TUNER = (OLD_SUBDEV_BASE + 5)
    IF_VID_DECODER = (BASE + 0x02001)
    IF_AUD_DECODER = (BASE + 0x02002)
    AUDIO_CAPTURE = (BASE + 0x03001)
    AUDIO_PLAYBACK = (BASE + 0x03002)
    AUDIO_MIXER = (BASE + 0x03003)
    PROC_VIDEO_COMPOSER = (BASE + 0x4001)
    PROC_VIDEO_PIXEL_FORMATTER = (BASE + 0x4002)
    PROC_VIDEO_PIXEL_ENC_CONV = (BASE + 0x4003)
    PROC_VIDEO_LUT = (BASE + 0x4004)
    PROC_VIDEO_SCALER = (BASE + 0x4005)
    PROC_VIDEO_STATISTICS = (BASE + 0x4006)
    PROC_VIDEO_ENCODER = (BASE + 0x4007)
    PROC_VIDEO_DECODER = (BASE + 0x4008)
    VID_MUX = (BASE + 0x5001)
    VID_IF_BRIDGE = (BASE + 0x5002)
    ATV_DECODER = (OLD_SUBDEV_BASE + 4)
    DV_DECODER = (BASE + 0x6001)
    DV_ENCODER = (BASE + 0x6002)
    DTV_DECODER = DV_DECODER


class Media_Ent_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    DEFAULT = (1 << 0)
    CONNECTOR = (1 << 1)


class Media_Ent_Id(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    FLAG_NEXT = (1 << 31)


class Media_Entity_Desc(Structure):
    class _u(Union):
        class _dev(Structure):
            _fields_ = [('major', c_uint32), ('minor', c_uint32)]

        class _alsa(Structure):
            _fields_ = [('card', c_uint32), ('device', c_uint32),
                        ('subdevice', c_uint32)]

        class _v4l(Structure):
            _fields_ = [('major', c_uint32), ('minor', c_uint32)]

        class _fb(Structure):
            _fields_ = [('major', c_uint32), ('minor', c_uint32)]

        _fields_ = [('dev', _dev), ('alsa', _alsa), ('v4l', _v4l), ('fb', _fb),
                    ('dvb', c_int), ('raw', c_uint8 * 184)]

    _fields_ = [
        ('id', c_uint32),
        ('name', c_char * 32),
        ('type', c_uint32),
        ('revision', c_uint32),
        ('flags', c_uint32),
        ('group_id', c_uint32),
        ('pads', c_uint16),
        ('links', c_uint16),
        ('reserved', c_uint32 * 4),
    ]


class Media_Pad_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    SINK = (1 << 0)
    SOURCE = (1 << 1)
    MUST_CONNECT = (1 << 2)


class Media_Pad_Desc(Structure):
    _fields_ = [('entity', c_uint32), ('index', c_uint16), ('flags', c_uint32),
                ('reserved', c_uint32 * 2)]


class Media_Link_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ENABLED = (1 << 0)
    IMMUTABLE = (1 << 1)
    DYNAMIC = (1 << 2)
    LINK_TYPE = (0xf << 28)
    DATA_LINK = (0 << 28)
    INTERFACE_LINK = (1 << 28)


class Media_Link_Desc(Structure):
    _fields_ = [('source', Media_Pad_Desc), ('sink', Media_Pad_Desc),
                ('flags', c_uint32), ('reserved', c_uint32 * 2)]


class Media_Links_Enum(Structure):
    _fields_ = [('entity', c_uint32), ('pads', POINTER(Media_Pad_Desc)),
                ('links', POINTER(Media_Link_Desc)), ('reserved', c_uint32)]


class Media_Intf_T(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    DVB_BASE = 0x00000100
    V4L_BASE = 0x00000200
    DVB_FE = (DVB_BASE)
    DVB_DEMUX = (DVB_BASE + 1)
    DVB_DVR = (DVB_BASE + 2)
    DVB_CA = (DVB_BASE + 3)
    DVB_NET = (DVB_BASE + 4)
    V4L_VIDEO = (V4L_BASE)
    V4L_VBI = (V4L_BASE + 1)
    V4L_RADIO = (V4L_BASE + 2)
    V4L_SUBDEV = (V4L_BASE + 3)
    V4L_SWRADIO = (V4L_BASE + 4)
    V4L_TOUCH = (V4L_BASE + 5)
    ALSA_BASE = 0x00000300
    ALSA_PCM_CAPTURE = (ALSA_BASE)
    ALSA_PCM_PLAYBACK = (ALSA_BASE + 1)
    ALSA_CONTROL = (ALSA_BASE + 2)
    ALSA_COMPRESS = (ALSA_BASE + 3)
    ALSA_RAWMIDI = (ALSA_BASE + 4)
    ALSA_HWDEP = (ALSA_BASE + 5)
    ALSA_SEQUENCER = (ALSA_BASE + 6)
    ALSA_TIMER = (ALSA_BASE + 7)


def media_v2_entity_has_flags(media_version):
    return ((media_version) >= ((4 << 16) | (19 << 8) | 0))


class Media_V2_Entity(Structure):
    _fields_ = [('id', c_uint32), ('name', c_char), ('function', c_uint32),
                ('flags', c_uint32), ('reserved', c_uint32 * 5)]
    _pack_ = True


class Media_V2_Intf_Devnode(Structure):
    _fields_ = [('major', c_uint32), ('minor', c_uint32)]
    _pack_ = True


class Media_V2_Interface(Structure):
    class _u(Union):
        _fields_ = [('devnode', Media_V2_Intf_Devnode), ('raw', c_uint32 * 16)]

    _fields_ = [('id', c_uint32), ('intf_type', c_uint32), ('flags', c_uint32),
                ('reserved', c_uint32)]
    _pack_ = True


def media_v2_pad_has_index(media_version):
    return ((media_version) >= ((4 << 16) | (19 << 8) | 0))


class Media_V2_Pad(Structure):
    _fields_ = [('id', c_uint32), ('entity_id', c_uint32), ('flags', c_uint32),
                ('index', c_uint32), ('reserved', c_uint32 * 4)]
    _pack_ = True


class Media_V2_Link(Structure):
    _fields_ = [
        ('id', c_uint32),
        ('source_id', c_uint32),
        ('sink_id', c_uint32),
        ('flags', c_uint32),
        ('reserved', c_uint32 * 6),
    ]
    _pack_ = True


class Media_V2_Topology(Structure):
    _fields_ = [('topology_version', c_uint64), ('num_entities', c_uint32),
                ('reserved1', c_uint32), ('ptr_entities', c_uint64),
                ('num_interfaces', c_uint32), ('reserved2', c_uint32),
                ('ptr_interfaces', c_uint64), ('num_pads', c_uint32),
                ('reserved3', c_uint32), ('ptr_pads', c_uint64),
                ('num_links', c_uint32), ('reserved4', c_uint32),
                ('ptr_links', c_uint64)]
    _pack_ = True


class Media_Ioc(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    DEVICE_INFO = iowr('|', 0x00, Media_Device_Info)
    ENUM_ENTITIES = iowr('|', 0x01, Media_Entity_Desc)
    ENUM_LINKS = iowr('|', 0x02, Media_Links_Enum)
    SETUP_LINK = iowr('|', 0x03, Media_Link_Desc)
    G_TOPOLOGY = iowr('|', 0x04, Media_V2_Topology)
    REQUEST_ALLOC = ior('|', 0x05, c_int)


class Media_Request_Ioc(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    QUEUE = io('|', 0x80)
    REINIT = io('|', 0x81)


class Media_Ent(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    TYPE_SHIFT = 16
    TYPE_MASK = 0x00ff0000
    SUBTYPE_MASK = 0x0000ffff


class Media_Ent_T(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    DEVNODE_UNKNOWN = (Media_Ent_F.OLD_BASE | Media_Ent.SUBTYPE_MASK)
    DEVNODE = Media_Ent_F.OLD_BASE
    DEVNODE_V4L = Media_Ent_F.IO_V4L
    DEVNODE_FB = (Media_Ent_F.OLD_BASE + 2)
    DEVNODE_ALSA = (Media_Ent_F.OLD_BASE + 3)
    DEVNODE_DVB = (Media_Ent_F.OLD_BASE + 4)
    UNKNOWN = Media_Ent_F.UNKNOWN
    V4L2_VIDEO = Media_Ent_F.IO_V4L
    V4L2_SUBDEV = Media_Ent_F.V4L2_SUBDEV_UNKNOWN
    V4L2_SUBDEV_SENSOR = Media_Ent_F.CAM_SENSOR
    V4L2_SUBDEV_FLASH = Media_Ent_F.FLASH
    V4L2_SUBDEV_LENS = Media_Ent_F.LENS
    V4L2_SUBDEV_DECODER = Media_Ent_F.ATV_DECODER
    V4L2_SUBDEV_TUNER = Media_Ent_F.TUNER


class Media_Api(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    VERSION = ((0 << 16) | (1 << 8) | 0)



from ctypes import *
from enum import IntEnum


def c_type(arg):
    return arg.__c_type__()


class Ioc(IntEnum):
    def __c_type__():
        return c_int32

    def __str__(self):
        return '{0}'.format(self.value)

    NRBITS = 8
    TYPEBITS = 8
    SIZEBITS = 14
    DIRBITS = 2

    NRMASK = ((1 << NRBITS) - 1)
    TYPEMASK = ((1 << TYPEBITS) - 1)
    SIZEMASK = ((1 << SIZEBITS) - 1)
    DIRMASK = ((1 << DIRBITS) - 1)

    NRSHIFT = 0
    TYPESHIFT = NRSHIFT + NRBITS
    SIZESHIFT = TYPESHIFT + TYPEBITS
    DIRSHIFT = SIZESHIFT + SIZEBITS

    NONE = 0
    WRITE = 1
    READ = 2

    IN = (WRITE << DIRSHIFT)
    OUT = (READ << DIRSHIFT)
    INOUT = ((WRITE | READ) << DIRSHIFT)
    SIZE_MASK = (SIZEMASK << SIZESHIFT)
    SIZE_SHIFT = (SIZESHIFT)


def ioc(dir_, type_, nr, size):
    return ((dir_ << Ioc.DIRSHIFT) | (ord(type_) << Ioc.TYPESHIFT) |
            (nr << Ioc.NRSHIFT) | (size << Ioc.SIZESHIFT))


def ioc_typecheck(t):
    return sizeof(t)


def io(type_, nr):
    return ioc(Ioc.NONE, type_, nr, 0)


def iow(type_, nr, size):
    return ioc(Ioc.WRITE, type_, nr, ioc_typecheck(size))


def ior(type_, nr, size):
    return ioc(Ioc.READ, type_, nr, ioc_typecheck(size))


def iowr(type_, nr, size):
    return ioc(Ioc.READ | Ioc.WRITE, type_, nr, ioc_typecheck(size))


def ior_bad(type_, nr, size):
    ioc(Ioc.READ, (type_), (nr), sizeof(size))


def iow_bad(type_, nr, size):
    ioc(Ioc.WRITE, (type_), (nr), sizeof(size))


def iowr_bad(type_, nr, size):
    ioc(Ioc.READ | Ioc.WRITE, (type_), (nr), sizeof(size))


def ioc_dir(nr):
    (((nr) >> Ioc.DIRSHIFT) & Ioc.DIRMASK)


def ioc_type(nr):
    (((nr) >> Ioc.TYPESHIFT) & Ioc.TYPEMASK)


def ioc_nr(nr):
    (((nr) >> Ioc.NRSHIFT) & Ioc.NRMASK)


def ioc_size(nr):
    (((nr) >> Ioc.SIZESHIFT) & Ioc.SIZEMASK)



from ctypes import *

def _or(self, other):
    if hasattr(other, 'value'):
        other = other.value
    return c_uint64(self.value | other)


def _coerce(self, other):
    try:
        return self, self.__class__(other)
    except TypeError:
        return NotImplemented


c_uint64.__or__ = _or
c_uint64.__coerce__ = _coerce

def c_type(arg):
    return arg.__c_type__()


from ctypes import *
from enum import Enum, IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.common import *


class Video_Max(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    FRAME = 32
    PLANES = 8


def v4l2_fourcc(a, b, c, d):
    return ord(a) | (ord(b) << 8) | (ord(c) << 16) | (ord(d) << 24)


def v4l2_fourcc_be(a, b, c, d):
    return v4l2_fourcc(a, b, c, d) | (1 << 31)


class V4l2_Field(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (
        ANY,
        NONE,
        TOP,
        BOTTOM,
        INTERLACED,
        SEQ_TB,
        SEQ_BT,
        ALTERNATE,
        INTERLACED_TB,
        INTERLACED_BT,
    ) = range(10)


def v4l2_field_has_top(field):
    return (field == V4l2_Field.TOP or field == V4l2_Field.INTERLACED
            or field == V4l2_Field.INTERLACED_TB
            or field == V4l2_Field.INTERLACED_BT or field == V4l2_Field.SEQ_TB
            or field == V4l2_Field.SEQ_BT)


def v4l2_field_has_bottom(field):
    return (field == V4l2_Field.BOTTOM or field == V4l2_Field.INTERLACED
            or field == V4l2_Field.INTERLACED_TB
            or field == V4l2_Field.INTERLACED_BT or field == V4l2_Field.SEQ_TB
            or field == V4l2_Field.SEQ_BT)


def v4l2_field_has_both(field):
    return (field == V4l2_Field.INTERLACED or field == V4l2_Field.INTERLACED_TB
            or field == V4l2_Field.INTERLACED_BT or field == V4l2_Field.SEQ_TB
            or field == V4l2_Field.SEQ_BT)


def v4l2_field_has_t_or_b(field):
    return (field == V4l2_Field.BOTTOM or field == V4l2_Field.TOP
            or field == V4l2_Field.ALTERNATE)


def v4l2_field_is_interlaced(field):
    return (field == V4l2_Field.INTERLACED or field == V4l2_Field.INTERLACED_TB
            or field == V4l2_Field.INTERLACED_BT)


def v4l2_field_is_sequential(field):
    return (field == V4l2_Field.SEQ_TB or field == V4l2_Field.SEQ_BT)


class V4l2_Buf_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (VIDEO_CAPTURE, VIDEO_OUTPUT, VIDEO_OVERLAY, VBI_CAPTURE, VBI_OUTPUT,
     SLICED_VBI_CAPTURE, SLICED_VBI_OUTPUT, VIDEO_OUTPUT_OVERLAY,
     VIDEO_CAPTURE_MPLANE, VIDEO_OUTPUT_MPLANE, SDR_CAPTURE, SDR_OUTPUT,
     META_CAPTURE, META_OUTPUT, PRIVATE) = list(range(1, 15)) + [0x80]


def v4l2_type_is_multiplanar(type):
    return (type == V4l2_Buf_Type.VIDEO_CAPTURE_MPLANE
            or type == V4l2_Buf_Type.VIDEO_OUTPUT_MPLANE)


def v4l2_type_is_output(type):
    return (type == V4l2_Buf_Type.VIDEO_OUTPUT
            or type == V4l2_Buf_Type.VIDEO_OUTPUT_MPLANE
            or type == V4l2_Buf_Type.VIDEO_OVERLAY
            or type == V4l2_Buf_Type.VIDEO_OUTPUT_OVERLAY
            or type == V4l2_Buf_Type.VBI_OUTPUT
            or type == V4l2_Buf_Type.SLICED_VBI_OUTPUT
            or type == V4l2_Buf_Type.SDR_OUTPUT
            or type == V4l2_Buf_Type.META_OUTPUT)


class V4l2_Tuner_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (RADIO, ANALOG_TV, DIGITAL_TV, SDR, RF) = range(1, 6)
    ADC = SDR


class V4l2_Memory(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (
        MMAP,
        USERPTR,
        OVERLAY,
        DMABUF,
    ) = range(1, 5)


class V4l2_Colorspace(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DEFAULT, SMPTE170M, SMPTE240M, REC709, BT878, C470_SYSTEM_M,
     C470_SYSTEM_BG, JPEG, SRGB, OPRGB, BT2020, RAW, DCI_P3) = range(13)


def v4l2_map_colorspace_default(is_sdtv, is_hdtv):
    return ((V4l2_Colorspace.SRGB, V4l2_Colorspace.REC709)[is_hdtv],
            V4l2_Colorspace.SMPTE170M)[is_sdtv]


class V4l2_Xfer_Func(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DEFAULT, X709, SRGB, OPRGB, SMPTE240M, NONE, DCI_P3, SMPTE2084) = range(8)


def v4l2_map_xfer_func_default(colsp):
    return (((((V4l2_Xfer_Func.X709,
                V4l2_Xfer_Func.SRGB)[colsp == V4l2_Colorspace.SRGB
                                     or colsp == V4l2_Colorspace.JPEG],
               V4l2_Xfer_Func.NONE)[colsp == V4l2_Colorspace.RAW],
              V4l2_Xfer_Func.DCI_P3)[colsp == V4l2_Colorspace.DCI_P3],
             V4l2_Xfer_Func.SMPTE240M)[colsp == V4l2_Colorspace.SMPTE240M],
            V4l2_Xfer_Func.OPRGB)[colsp == V4l2_Colorspace.OPRGB]


class V4l2_Ycbcr_Enc(IntEnum):
    def __c_type__(IntEnum):
        return c_uint32

    def __str__(self):
        return '{0}'.format(self.value)

    (DEFAULT, Y601, Y709, XV601, XV709, SYCC, BT2020, BT2020_CONST_LUM,
     SMPTE240M) = range(9)


class V4l2_Hsv_Enc(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (H180, H256) = range(128, 130)


def v4l2_map_ycbcr_enc_default(colsp):
    return (((V4l2_Ycbcr_Enc.Y601,
              V4l2_Ycbcr_Enc.SMPTE240M)[colsp == V4l2_Colorspace.SMPTE240M],
             V4l2_Ycbcr_Enc.BT2020)[colsp == V4l2_Colorspace.BT2020],
            V4l2_Ycbcr_Enc.Y709)[colsp == V4l2_Colorspace.REC709
                                 or colsp == V4l2_Colorspace.DCI_P3]


class V4l2_Quantization(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DEFAULT, FULL_RANGE, LIM_RANGE) = range(3)


def v4l2_map_quantization_default(is_rgb_or_hsv, colsp, ycbcr_enc):
    return ((V4l2_Quantization.LIM_RANGE,
             V4l2_Quantization.FULL_RANGE)[((is_rgb_or_hsv) or
                                            (colsp) == V4l2_Colorspace.JPEG)],
            V4l2_Quantization.LIM_RANGE)[((is_rgb_or_hsv) and
                                          (colsp) == V4l2_Colorspace.BT2020)]


class V4l2_Priority(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (
        UNSET,
        BACKGROUND,
        INTERACTIVE,
        RECORD,
    ) = range(0, 4)
    DEFAULT = INTERACTIVE


class V4l2_Rect(Structure):
    _fields_ = [
        ('left', c_int32),
        ('top', c_int32),
        ('width', c_uint32),
        ('height', c_uint32),
    ]


class V4l2_Fract(Structure):
    _fields_ = [
        ('numerator', c_uint32),
        ('denominator', c_uint32),
    ]


class V4l2_Area(Structure):
    _fields_ = [
        ('width', c_uint32),
        ('height', c_uint32),
    ]


class V4l2_Capability(Structure):
    _fields_ = [('driver', c_uint8 * 16), ('card', c_uint8 * 32),
                ('bus_info', c_uint8 * 32), ('version', c_uint32),
                ('capabilities', c_uint32), ('device_caps', c_uint32),
                ('reserved', c_uint32 * 3)]


class V4l2_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    HIGHQUALITY = 0 << 1


class V4l2_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (VIDEO_CAPTURE, VIDEO_OUTPUT, VIDEO_OVERLAY, VBI_CAPTURE, VBI_OUTPUT,
     SLICED_VBI_CAPTURE, SLICED_VBI_OUTPUT, RDS_CAPTURE, VIDEO_OUTPUT_OVERLAY,
     HW_FREQ_SEEK, RDS_OUTPUT, VIDEO_CAPTURE_MPLANE, VIDEO_OUTPUT_MPLANE,
     VIDEO_M2M_MPLANE, VIDEO_M2M, TUNER, AUDIO, RADIO, MODULATOR, SDR_CAPTURE,
     EXT_PIX_FORMAT, SDR_OUTPUT, META_CAPTURE, READWRITE, ASYNCIO, STREAMING,
     META_OUTPUT, TOUCH, IO_MC) = [1 << x for x in range(29)]
    DEVICE_CAPS = 1 << 30
    TIMEPERFRAME = 1 << 13


class V4l2_Fmt(Structure):
    _fields_ = [('width', c_uint32), ('height', c_uint32),
                ('pixelformat', c_uint32), ('field', c_uint32),
                ('bytesperline', c_uint32), ('sizeimage', c_uint32),
                ('colorspace', c_uint32), ('priv', c_uint32)]


class V4l2_Fmt_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (COMPRESSED, EMULATED, CONTINUOUS_BYTESTREAM,
     DYN_RESOLUTION) = [1 << x for x in range(4)]


class V4l2_Pix_Format(Structure):
    class _u(Union):
        _fields_ = [
            ('ycbcr_enc', c_uint32),
            ('hsv_enc', c_uint32),
        ]

    _fields_ = [
        ('width', c_uint32),
        ('height', c_uint32),
        ('pixelformat', c_uint32),
        ('field', c_uint32),
        ('bytesperline', c_uint32),
        ('sizeimage', c_uint32),
        ('colorspace', c_uint32),
        ('priv', c_uint32),
        ('flags', c_uint32),
        ('_u', _u),
        ('quantization', c_uint32),
        ('xfer_func', c_uint32),
    ]


class V4l2_Pix_Fmt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    RGB332 = v4l2_fourcc('R', 'G', 'B', '1')
    RGB444 = v4l2_fourcc('R', '4', '4', '4')
    ARGB444 = v4l2_fourcc('A', 'R', '1', '2')
    XRGB444 = v4l2_fourcc('X', 'R', '1', '2')
    RGBA444 = v4l2_fourcc('R', 'A', '1', '2')
    RGBX444 = v4l2_fourcc('R', 'X', '1', '2')
    ABGR444 = v4l2_fourcc('A', 'B', '1', '2')
    XBGR444 = v4l2_fourcc('X', 'B', '1', '2')
    BGRX444 = v4l2_fourcc('B', 'X', '1', '2')
    RGB555 = v4l2_fourcc('R', 'G', 'B', 'O')
    ARGB555 = v4l2_fourcc('A', 'R', '1', '5')
    XRGB555 = v4l2_fourcc('X', 'R', '1', '5')
    RGBA555 = v4l2_fourcc('R', 'A', '1', '5')
    RGBX555 = v4l2_fourcc('R', 'X', '1', '5')
    ABGR555 = v4l2_fourcc('A', 'B', '1', '5')
    XBGR555 = v4l2_fourcc('X', 'B', '1', '5')
    BGRA555 = v4l2_fourcc('B', 'A', '1', '5')
    BGRX555 = v4l2_fourcc('B', 'X', '1', '5')
    RGB565 = v4l2_fourcc('R', 'G', 'B', 'P')
    RGB555X = v4l2_fourcc('R', 'G', 'B', 'Q')
    ARGB555X = v4l2_fourcc_be('A', 'R', '1', '5')
    XRGB555X = v4l2_fourcc_be('X', 'R', '1', '5')
    RGB565X = v4l2_fourcc('R', 'G', 'B', 'R')
    BGR666 = v4l2_fourcc('B', 'G', 'R', 'H')
    BGR24 = v4l2_fourcc('B', 'G', 'R', '3')
    RGB24 = v4l2_fourcc('R', 'G', 'B', '3')
    BGR32 = v4l2_fourcc('B', 'G', 'R', '4')
    ABGR32 = v4l2_fourcc('A', 'R', '2', '4')
    XBGR32 = v4l2_fourcc('X', 'R', '2', '4')
    BGRA32 = v4l2_fourcc('R', 'A', '2', '4')
    BGRX32 = v4l2_fourcc('R', 'X', '2', '4')
    RGB32 = v4l2_fourcc('R', 'G', 'B', '4')
    RGBA32 = v4l2_fourcc('A', 'B', '2', '4')
    RGBX32 = v4l2_fourcc('X', 'B', '2', '4')
    ARGB32 = v4l2_fourcc('B', 'A', '2', '4')
    XRGB32 = v4l2_fourcc('B', 'X', '2', '4')
    GREY = v4l2_fourcc('G', 'R', 'E', 'Y')
    Y4 = v4l2_fourcc('Y', '0', '4', ' ')
    Y6 = v4l2_fourcc('Y', '0', '6', ' ')
    Y10 = v4l2_fourcc('Y', '1', '0', ' ')
    Y12 = v4l2_fourcc('Y', '1', '2', ' ')
    Y14 = v4l2_fourcc('Y', '1', '4', ' ')
    Y16 = v4l2_fourcc('Y', '1', '6', ' ')
    Y16_BE = v4l2_fourcc_be('Y', '1', '6', ' ')
    Y10BPACK = v4l2_fourcc('Y', '1', '0', 'B')
    Y10P = v4l2_fourcc('Y', '1', '0', 'P')
    PAL8 = v4l2_fourcc('P', 'A', 'L', '8')
    UV8 = v4l2_fourcc('U', 'V', '8', ' ')
    YUYV = v4l2_fourcc('Y', 'U', 'Y', 'V')
    YYUV = v4l2_fourcc('Y', 'Y', 'U', 'V')
    YVYU = v4l2_fourcc('Y', 'V', 'Y', 'U')
    UYVY = v4l2_fourcc('U', 'Y', 'V', 'Y')
    VYUY = v4l2_fourcc('V', 'Y', 'U', 'Y')
    Y41P = v4l2_fourcc('Y', '4', '1', 'P')
    YUV444 = v4l2_fourcc('Y', '4', '4', '4')
    YUV555 = v4l2_fourcc('Y', 'U', 'V', 'O')
    YUV565 = v4l2_fourcc('Y', 'U', 'V', 'P')
    YUV32 = v4l2_fourcc('Y', 'U', 'V', '4')
    AYUV32 = v4l2_fourcc('A', 'Y', 'U', 'V')
    XYUV32 = v4l2_fourcc('X', 'Y', 'U', 'V')
    VUYA32 = v4l2_fourcc('V', 'U', 'Y', 'A')
    VUYX32 = v4l2_fourcc('V', 'U', 'Y', 'X')
    HI240 = v4l2_fourcc('H', 'I', '2', '4')
    HM12 = v4l2_fourcc('H', 'M', '1', '2')
    M420 = v4l2_fourcc('M', '4', '2', '0')
    NV12 = v4l2_fourcc('N', 'V', '1', '2')
    NV21 = v4l2_fourcc('N', 'V', '2', '1')
    NV16 = v4l2_fourcc('N', 'V', '1', '6')
    NV61 = v4l2_fourcc('N', 'V', '6', '1')
    NV24 = v4l2_fourcc('N', 'V', '2', '4')
    NV42 = v4l2_fourcc('N', 'V', '4', '2')
    NV12M = v4l2_fourcc('N', 'M', '1', '2')
    NV21M = v4l2_fourcc('N', 'M', '2', '1')
    NV16M = v4l2_fourcc('N', 'M', '1', '6')
    NV61M = v4l2_fourcc('N', 'M', '6', '1')
    NV12MT = v4l2_fourcc('T', 'M', '1', '2')
    NV12MT_16X16 = v4l2_fourcc('V', 'M', '1', '2')
    YUV410 = v4l2_fourcc('Y', 'U', 'V', '9')
    YVU410 = v4l2_fourcc('Y', 'V', 'U', '9')
    YUV411P = v4l2_fourcc('4', '1', '1', 'P')
    YUV420 = v4l2_fourcc('Y', 'U', '1', '2')
    YVU420 = v4l2_fourcc('Y', 'V', '1', '2')
    YUV422P = v4l2_fourcc('4', '2', '2', 'P')
    YUV420M = v4l2_fourcc('Y', 'M', '1', '2')
    YVU420M = v4l2_fourcc('Y', 'M', '2', '1')
    YUV422M = v4l2_fourcc('Y', 'M', '1', '6')
    YVU422M = v4l2_fourcc('Y', 'M', '6', '1')
    YUV444M = v4l2_fourcc('Y', 'M', '2', '4')
    YVU444M = v4l2_fourcc('Y', 'M', '4', '2')
    SBGGR8 = v4l2_fourcc('B', 'A', '8', '1')
    SGBRG8 = v4l2_fourcc('G', 'B', 'R', 'G')
    SGRBG8 = v4l2_fourcc('G', 'R', 'B', 'G')
    SRGGB8 = v4l2_fourcc('R', 'G', 'G', 'B')
    SBGGR10 = v4l2_fourcc('B', 'G', '1', '0')
    SGBRG10 = v4l2_fourcc('G', 'B', '1', '0')
    SGRBG10 = v4l2_fourcc('B', 'A', '1', '0')
    SRGGB10 = v4l2_fourcc('R', 'G', '1', '0')
    SBGGR10P = v4l2_fourcc('p', 'B', 'A', 'A')
    SGBRG10P = v4l2_fourcc('p', 'G', 'A', 'A')
    SGRBG10P = v4l2_fourcc('p', 'g', 'A', 'A')
    SRGGB10P = v4l2_fourcc('p', 'R', 'A', 'A')
    SBGGR10ALAW8 = v4l2_fourcc('a', 'B', 'A', '8')
    SGBRG10ALAW8 = v4l2_fourcc('a', 'G', 'A', '8')
    SGRBG10ALAW8 = v4l2_fourcc('a', 'g', 'A', '8')
    SRGGB10ALAW8 = v4l2_fourcc('a', 'R', 'A', '8')
    SBGGR10DPCM8 = v4l2_fourcc('b', 'B', 'A', '8')
    SGBRG10DPCM8 = v4l2_fourcc('b', 'G', 'A', '8')
    SGRBG10DPCM8 = v4l2_fourcc('B', 'D', '1', '0')
    SRGGB10DPCM8 = v4l2_fourcc('b', 'R', 'A', '8')
    SBGGR12 = v4l2_fourcc('B', 'G', '1', '2')
    SGBRG12 = v4l2_fourcc('G', 'B', '1', '2')
    SGRBG12 = v4l2_fourcc('B', 'A', '1', '2')
    SRGGB12 = v4l2_fourcc('R', 'G', '1', '2')
    SBGGR12P = v4l2_fourcc('p', 'B', 'C', 'C')
    SGBRG12P = v4l2_fourcc('p', 'G', 'C', 'C')
    SGRBG12P = v4l2_fourcc('p', 'g', 'C', 'C')
    SRGGB12P = v4l2_fourcc('p', 'R', 'C', 'C')
    SBGGR14 = v4l2_fourcc('B', 'G', '1', '4')
    SGBRG14 = v4l2_fourcc('G', 'B', '1', '4')
    SGRBG14 = v4l2_fourcc('G', 'R', '1', '4')
    SRGGB14 = v4l2_fourcc('R', 'G', '1', '4')
    SBGGR14P = v4l2_fourcc('p', 'B', 'E', 'E')
    SGBRG14P = v4l2_fourcc('p', 'G', 'E', 'E')
    SGRBG14P = v4l2_fourcc('p', 'g', 'E', 'E')
    SRGGB14P = v4l2_fourcc('p', 'R', 'E', 'E')
    SBGGR16 = v4l2_fourcc('B', 'Y', 'R', '2')
    SGBRG16 = v4l2_fourcc('G', 'B', '1', '6')
    SGRBG16 = v4l2_fourcc('G', 'R', '1', '6')
    SRGGB16 = v4l2_fourcc('R', 'G', '1', '6')
    HSV24 = v4l2_fourcc('H', 'S', 'V', '3')
    HSV32 = v4l2_fourcc('H', 'S', 'V', '4')
    MJPEG = v4l2_fourcc('M', 'J', 'P', 'G')
    JPEG = v4l2_fourcc('J', 'P', 'E', 'G')
    DV = v4l2_fourcc('d', 'v', 's', 'd')
    MPEG = v4l2_fourcc('M', 'P', 'E', 'G')
    H264 = v4l2_fourcc('H', '2', '6', '4')
    H264_NO_SC = v4l2_fourcc('A', 'V', 'C', '1')
    H264_MVC = v4l2_fourcc('M', '2', '6', '4')
    H263 = v4l2_fourcc('H', '2', '6', '3')
    MPEG1 = v4l2_fourcc('M', 'P', 'G', '1')
    MPEG2 = v4l2_fourcc('M', 'P', 'G', '2')
    MPEG2_SLICE = v4l2_fourcc('M', 'G', '2', 'S')
    MPEG4 = v4l2_fourcc('M', 'P', 'G', '4')
    XVID = v4l2_fourcc('X', 'V', 'I', 'D')
    VC1_ANNEX_G = v4l2_fourcc('V', 'C', '1', 'G')
    VC1_ANNEX_L = v4l2_fourcc('V', 'C', '1', 'L')
    VP8 = v4l2_fourcc('V', 'P', '8', '0')
    VP9 = v4l2_fourcc('V', 'P', '9', '0')
    HEVC = v4l2_fourcc('H', 'E', 'V', 'C')
    FWHT = v4l2_fourcc('F', 'W', 'H', 'T')
    FWHT_STATELESS = v4l2_fourcc('S', 'F', 'W', 'H')
    CPIA1 = v4l2_fourcc('C', 'P', 'I', 'A')
    WNVA = v4l2_fourcc('W', 'N', 'V', 'A')
    SN9C10X = v4l2_fourcc('S', '9', '1', '0')
    SN9C20X_I420 = v4l2_fourcc('S', '9', '2', '0')
    PWC1 = v4l2_fourcc('P', 'W', 'C', '1')
    PWC2 = v4l2_fourcc('P', 'W', 'C', '2')
    ET61X251 = v4l2_fourcc('E', '6', '2', '5')
    SPCA501 = v4l2_fourcc('S', '5', '0', '1')
    SPCA505 = v4l2_fourcc('S', '5', '0', '5')
    SPCA508 = v4l2_fourcc('S', '5', '0', '8')
    SPCA561 = v4l2_fourcc('S', '5', '6', '1')
    PAC207 = v4l2_fourcc('P', '2', '0', '7')
    MR97310A = v4l2_fourcc('M', '3', '1', '0')
    JL2005BCD = v4l2_fourcc('J', 'L', '2', '0')
    SN9C2028 = v4l2_fourcc('S', 'O', 'N', 'X')
    SQ905C = v4l2_fourcc('9', '0', '5', 'C')
    PJPG = v4l2_fourcc('P', 'J', 'P', 'G')
    OV511 = v4l2_fourcc('O', '5', '1', '1')
    OV518 = v4l2_fourcc('O', '5', '1', '8')
    STV0680 = v4l2_fourcc('S', '6', '8', '0')
    TM6000 = v4l2_fourcc('T', 'M', '6', '0')
    CIT_YYVYUY = v4l2_fourcc('C', 'I', 'T', 'V')
    KONICA420 = v4l2_fourcc('K', 'O', 'N', 'I')
    JPGL = v4l2_fourcc('J', 'P', 'G', 'L')
    SE401 = v4l2_fourcc('S', '4', '0', '1')
    S5C_UYVY_JPG = v4l2_fourcc('S', '5', 'C', 'I')
    Y8I = v4l2_fourcc('Y', '8', 'I', ' ')
    Y12I = v4l2_fourcc('Y', '1', '2', 'I')
    Z16 = v4l2_fourcc('Z', '1', '6', ' ')
    MT21C = v4l2_fourcc('M', 'T', '2', '1')
    INZI = v4l2_fourcc('I', 'N', 'Z', 'I')
    SUNXI_TILED_NV12 = v4l2_fourcc('S', 'T', '1', '2')
    CNF4 = v4l2_fourcc('C', 'N', 'F', '4')
    IPU3_SBGGR10 = v4l2_fourcc('i', 'p', '3', 'b')
    IPU3_SGBRG10 = v4l2_fourcc('i', 'p', '3', 'g')
    IPU3_SGRBG10 = v4l2_fourcc('i', 'p', '3', 'G')
    IPU3_SRGGB10 = v4l2_fourcc('i', 'p', '3', 'r')
    PRIV_MAGIC = 0xfeedcafe
    FLAG_PREMUL_ALPHA = 0x00000001


class V4l2_Sdr_Fmt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    CU8 = v4l2_fourcc('C', 'U', '0', '8')
    CU16LE = v4l2_fourcc('C', 'U', '1', '6')
    CS8 = v4l2_fourcc('C', 'S', '0', '8')
    CS14LE = v4l2_fourcc('C', 'S', '1', '4')
    RU12LE = v4l2_fourcc('R', 'U', '1', '2')
    PCU16BE = v4l2_fourcc('P', 'C', '1', '6')
    PCU18BE = v4l2_fourcc('P', 'C', '1', '8')
    PCU20BE = v4l2_fourcc('P', 'C', '2', '0')


class V4l2_Tch_Fmt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    DELTA_TD16 = v4l2_fourcc('T', 'D', '1', '6')
    DELTA_TD08 = v4l2_fourcc('T', 'D', '0', '8')
    TU16 = v4l2_fourcc('T', 'U', '1', '6')
    TU08 = v4l2_fourcc('T', 'U', '0', '8')


class V4l2_Meta_Fmt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    VSP1_HGO = v4l2_fourcc('V', 'S', 'P', 'H')
    VSP1_HGT = v4l2_fourcc('V', 'S', 'P', 'T')
    UVC = v4l2_fourcc('U', 'V', 'C', 'H')
    D4XX = v4l2_fourcc('D', '4', 'X', 'X')
    VIVID = v4l2_fourcc('V', 'I', 'V', 'D')


class V4l2_Fmtdesc(Structure):
    _fields_ = [('index', c_uint32), ('type', c_uint32), ('flags', c_uint32),
                ('description', c_uint8 * 32), ('pixelformat', c_uint32),
                ('mbus_code', c_uint32), ('reserved', c_uint32 * 3)]


class V4l2_Frmsizetypes(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DISCRETE, CONTINUOUS, STEPWISE) = range(1, 4)


class V4l2_Frmsize_Discrete(Structure):
    _fields_ = [('width', c_uint32), ('height', c_uint32)]


class V4l2_Frmsize_Stepwise(Structure):
    _fields_ = [('min_width', c_uint32), ('max_width', c_uint32),
                ('step_width', c_uint32), ('min_height', c_uint32),
                ('max_height', c_uint32), ('step_height', c_uint32)]


class V4l2_Frmsizeenum(Structure):
    class _u(Union):
        _fields_ = [('discrete', V4l2_Frmsize_Discrete),
                    ('stepwise', V4l2_Frmsize_Stepwise)]

    _fields_ = [('index', c_uint32), ('pixel_format', c_uint32),
                ('type', c_uint32), ('_u', _u), ('reserved', c_uint32 * 2)]


class V4l2_Frmivaltypes(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DISCRETE, CONTINUOUS, STEPWISE) = range(1, 4)


class V4l2_Frmival_Stepwise(Structure):
    _fields_ = [('min', V4l2_Fract), ('max', V4l2_Fract), ('step', V4l2_Fract)]


class V4l2_Frmivalenum(Structure):
    class _u(Union):
        _fields_ = [('discrete', V4l2_Fract),
                    ('stepwise', V4l2_Frmsize_Stepwise)]

    _fields_ = [('index', c_uint32), ('pixel_format', c_uint32),
                ('width', c_uint32), ('height', c_uint32), ('type', c_uint32),
                ('_u', _u), ('reserved', c_uint32 * 2)]


class V4l2_Frmival_Stepwise(Structure):
    _fields_ = [('type', c_uint32), ('flags', c_uint32), ('frames', c_uint8),
                ('seconds', c_uint8), ('minutes', c_uint8), ('hours', c_uint8),
                ('userbits', c_uint8 * 4)]


class v4l2_Timecode(Structure):
    _fields_ = [('type', c_uint32), ('flags', c_uint32), ('frames', c_uint8),
                ('seconds', c_uint8), ('minutes', c_uint8), ('hours', c_uint8),
                ('userbits', c_uint8 * 4)]


class V4l2_Tc_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (T24FPS, T25FPS, T30FPS, T50FPS, T60FPS) = range(1, 6)


class V4l2_Tc_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DROPFRAME, COLORFRAME) = [1 << x for x in range(2)]


class V4l2_Tc_Userbits(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    field = 0x000C
    USERDEFINED = 0x0000
    T8BITCHARS = 0x0008


class V4l2_Jpeg_Marker(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DHT, DQT, DRI, COM, APP) = [1 << x for x in range(3, 8)]


class V4l2_Jpegcompression(Structure):
    _fields_ = [('quality', c_int), ('APPn', c_int), ('APP_len', c_int),
                ('APP_data', c_char * 60), ('COM_len', c_int),
                ('COM_data', c_char * 60), ('jpeg_markers', c_uint32)]


class V4l2_Requestbuffers(Structure):
    _fields_ = [('count', c_uint32), ('type', c_uint32), ('memory', c_uint32),
                ('capabilities', c_uint32), ('reserved', c_uint32)]


class V4l2_Buf_Cap_Supports(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (MMAP, USERPTR, DMABUF, REQUESTS, ORPHANED_BUFS,
     M2M_HOLD_CAPTURE_BUF) = [1 << x for x in range(6)]


class V4l2_Plane(Structure):
    class _u(Union):
        _fields_ = [('mem_offset', c_uint32), ('userptr', c_ulong),
                    ('fd', c_int32)]

    _fields_ = [('bytesused', c_uint32), ('length', c_uint32), ('_u', _u),
                ('data_offset', c_uint32), ('reserved', c_uint32 * 11)]


class Timeval(Structure):
    _fields_ = [
        ('secs', c_long),
        ('usecs', c_long),
    ]


class V4l2_Buffer(Structure):
    class _u(Union):
        _fields_ = [('offset', c_uint32), ('userptr', c_ulong),
                    ('planes', V4l2_Plane), ('fd', c_int32)]

    class _v(Union):
        _fields_ = [('request_fd', c_int32), ('reserved', c_uint32)]

    _fields_ = [('index', c_uint32), ('type', c_uint32),
                ('bytesused', c_uint32), ('flags', c_uint32),
                ('field', c_uint32), ('timestamp', Timeval),
                ('timecode', v4l2_Timecode), ('sequence', c_uint32),
                ('memory', c_uint32), ('_u', _u), ('length', c_uint32),
                ('reserved2', c_uint32), ('_v', _v)]


class V4l2_Buf_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (MAPPED, QUEUED, DONE, KEYFRAME, PFRAME, BFRAME, ERROR, IN_REQUEST,
     TIMECODE, M2M_HOLD_CAPTURE_BUF, PREPARED, NO_CACHE_INVALIDATE,
     NO_CACHE_CLEAN, TIMESTAMP_MONOTONIC,
     TIMESTAMP_COPY) = [1 << x for x in range(15)]
    TIMESTAMP_UNKNOWN = TSTAMP_SRC_EOF = 0x00000000
    TIMESTAMP_MASK = 0x0000e000
    TSTAMP_SRC_MASK = 0x00070000
    TSTAMP_SRC_SOE = 0x00010000
    LAST = 0x00100000
    REQUEST_FD = 0x00800000


class V4l2_Exportbuffer(Structure):
    _fields_ = [('type', c_uint32), ('index', c_uint32), ('plane', c_uint32),
                ('flags', c_uint32), ('fd', c_int32),
                ('reserved', c_uint32 * 11)]


class V4l2_Framebuffer(Structure):
    _fields_ = [('capability', c_uint32), ('flags', c_uint32),
                ('base', c_void_p), ('fmt', V4l2_Fmt)]


class V4l2_Fbuf_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (EXTERNOVERLAY, CHROMAKEY, LIST_CLIPPING, BITMAP_CLIPPING, LOCAL_ALPHA,
     GLOBAL_ALPHA, LOCAL_INV_ALPHA,
     SRC_CHROMAKEY) = [1 << x for x in range(8)]


class V4l2_Fbuf_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PRIMARY, OVERLAY, CHROMAKEY, LOCAL_ALPHA, GLOBAL_ALPHA, LOCAL_INV_ALPHA,
     SRC_CHROMAKEY) = [1 << x for x in range(7)]


class V4l2_Clip(Structure):
    _fields_ = [('c', V4l2_Rect)]


V4l2_Clip._fields_.append(('next', POINTER(V4l2_Clip)))


class V4l2_Window(Structure):
    _fields_ = [('w', V4l2_Rect), ('field', c_uint32), ('chromakey', c_uint32),
                ('clips', POINTER(V4l2_Clip)), ('clipcount', c_uint32),
                ('bitmap', c_void_p), ('global_alpha', c_uint8)]


class V4l2_Captureparm(Structure):
    _fields_ = [('capability', c_uint32), ('capturemode', c_uint32),
                ('timeperframe', V4l2_Fract), ('extendedmode', c_uint32),
                ('readbuffers', c_uint32), ('reserved', c_uint32 * 4)]


class V4l2_Outputparm(Structure):
    _fields_ = [('capability', c_uint32), ('outputmode', c_uint32),
                ('timeperframe', V4l2_Fract), ('extendedmode', c_uint32),
                ('writebuffers', c_uint32), ('reserved', c_uint32 * 4)]


class V4l2_Cropcap(Structure):
    _fields_ = [
        ('type', c_uint32),
        ('bounds', V4l2_Rect),
        ('defrect', V4l2_Rect),
        ('pixelaspect', V4l2_Fract),
    ]


class V4l2_Crop(Structure):
    _fields_ = [('type', c_uint32), ('c', V4l2_Rect)]


class V4l2_Selection(Structure):
    _fields_ = [('type', c_uint32), ('target', c_uint32), ('flags', c_uint32),
                ('r', V4l2_Rect), ('reserved', c_uint32)]


V4l2_Std_Id = c_uint64


class V4l2_Std(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (PAL_B, PAL_B1, PAL_G, PAL_H, PAL_I, PAL_D, PAL_D1, PAL_K, PAL_M, PAL_N,
     PAL_Nc, PAL_60, NTSC_M, NTSC_M_JP, NTSC_443, NTSC_M_KR, SECAM_B, SECAM_D,
     SECAM_G, SECAM_H, SECAM_K, SECAM_K1, SECAM_L, SECAM_LC, ATSC_8_VSB,
     ATSC_16_VSB) = [V4l2_Std_Id(1 << x).value for x in range(26)]

    NTSC = (NTSC_M | NTSC_M_JP | NTSC_M_KR)
    SECAM_DK = (SECAM_D | SECAM_K | SECAM_K1)
    SECAM = (SECAM_B | SECAM_G | SECAM_H | SECAM_DK | SECAM_L | SECAM_LC)
    PAL_BG = (PAL_B | PAL_B1 | PAL_G)
    PAL_DK = (PAL_D | PAL_D1 | PAL_K)
    PAL = (PAL_BG | PAL_DK | PAL_H | PAL_I)
    B = (PAL_B | PAL_B1 | SECAM_B)
    G = (PAL_G | SECAM_G)
    H = (PAL_H | SECAM_H)
    L = (SECAM_L | SECAM_LC)
    GH = (G | H)
    DK = (PAL_DK | SECAM_DK)
    BG = (B | G)
    MN = (PAL_M | PAL_N | PAL_Nc | NTSC)
    MTS = (NTSC_M | PAL_M | PAL_N | PAL_Nc)
    S525_60 = (PAL_M | PAL_60 | NTSC | NTSC_443)
    S625_50 = (PAL | PAL_N | PAL_Nc | SECAM)
    ATSC = (ATSC_8_VSB | ATSC_16_VSB)
    UNKNOWN = 0
    ALL = (S525_60 | S625_50)


class V4l2_Standard(Structure):
    _fields_ = [('index', c_uint32), ('id', V4l2_Std_Id),
                ('name', c_uint8 * 24), ('frameperiod', V4l2_Fract),
                ('framelines', c_uint32), ('reserved', c_uint32 * 4)]


class V4l2_Bt_Timings(Structure):
    _fields_ = [('width', c_uint32), ('height', c_uint32),
                ('interlaced', c_uint32), ('polarities', c_uint32),
                ('pixelclock', c_uint64), ('hfrontporch', c_uint32),
                ('hsync', c_uint32), ('hbackporch', c_uint32),
                ('vfrontporch', c_uint32), ('vsync', c_uint32),
                ('vbackporch', c_uint32), ('il_vfrontporch', c_uint32),
                ('il_vsync', c_uint32), ('il_vbackporch', c_uint32),
                ('standards', c_uint32), ('flags', c_uint32),
                ('picture_aspect', V4l2_Fract), ('cea861_vic', c_uint8),
                ('hdmi_vic', c_uint8), ('reserved', c_uint8 * 46)]
    _pack_ = True


class V4l2_Dv(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (VSYNC_POS_POL, HSYNC_POS_POL) = [1 << x for x in range(2)]


class V4l2_Dv_Bt_Std(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (CEA861, DMT, CVT, GTF, SDI) = [1 << x for x in range(5)]


class V4l2_Dv_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (REDUCED_BLANKING, CAN_REDUCE_FPS, REDUCED_FPS, HALF_LINE, IS_CE_VIDEO,
     FIRST_FIELD_EXTRA_LINE, HAS_PICTURE_ASPECT, HAS_CEA861_VIC, HAS_HDMI_VIC,
     CAN_DETECT_REDUCED_FPS) = [1 << x for x in range(10)]


def v4l2_dv_bt_blanking_width(bt: V4l2_Bt_Timings):
    return (bt.hfrontporch + bt.hsync + bt.hbackporch)


def v4l2_dv_bt_frame_width(bt: V4l2_Bt_Timings):
    return (bt.width + v4l2_dv_bt_blanking_width(bt))


def v4l2_dv_bt_blanking_height(bt: V4l2_Bt_Timings):
    return (bt.vfrontporch + bt.vsync + bt.vbackporch + bt.il_vfrontporch +
            bt.il_vsync + bt.il_vbackporch)


def v4l2_dv_bt_frame_height(bt: V4l2_Bt_Timings):
    return (bt.height + v4l2_dv_bt_blanking_height(bt))


class V4l2_Dv_Timings(Structure):
    _fields_ = [('bt', V4l2_Bt_Timings), ('reserved', c_uint32 * 32)]
    _pack_ = True


class V4l2_Dv_Bt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    D656_1120 = 0


class V4l2_Enum_Dv_Timings(Structure):
    _fields_ = [('index', c_uint32), ('pad', c_uint32),
                ('reserved', c_uint32 * 2), ('timings', V4l2_Dv_Timings)]
    _pack_ = True


class V4l2_Bt_Timings_Cap(Structure):
    _fields_ = [
        ('min_width', c_uint32),
        ('max_width', c_uint32),
        ('min_height', c_uint32),
        ('max_height', c_uint32),
        ('min_pixelclock', c_uint32),
        ('max_pixelclock', c_uint32),
        ('standards', c_uint32),
        ('capabilities', c_uint32),
        ('reserved', c_uint32 * 16),
    ]
    _pack_ = True


class V4l2_Dv_Bt_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (INTERLACED, PROGRESSIVE, REDUCED_BLANKING,
     CUSTOM) = [1 << x for x in range(4)]


class V4l2_Dv_Timings_Cap(Structure):
    class _u(Union):
        _fields_ = [('bt', V4l2_Bt_Timings), ('raw_data', c_uint32 * 32)]

    _fields_ = [
        ('type', c_uint32),
        ('pad', c_uint32),
        ('reserved', c_uint32 * 2),
        ('_u', _u),
    ]


class V4l2_Input(Structure):
    _fields_ = [('index', c_uint32), ('name', c_uint8), ('type', c_uint32),
                ('audioset', c_uint32), ('tuner', c_uint32),
                ('std', V4l2_Std_Id), ('status', c_uint32),
                ('capabilities', c_uint32), ('reserved', c_uint32 * 3)]


class V4l2_Input_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (TUNER, CAMERA, TOUCH) = range(3)


class V4l2_In_St(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (NO_POWER, NO_SIGNAL, NO_COLOR) = [1 << x for x in range(3)]
    (HFLIP, VFLIP) = [1 << x for x in range(4, 6)]
    (NO_H_LOCK, COLOR_KILL, NO_V_LOCK,
     NO_STD_LOCK) = [1 << x for x in range(8, 12)]
    (NO_SYNC, NO_EQU, NO_CARRIER) = [1 << x for x in range(16, 19)]
    (MACROVISION, NO_ACCESS, VTR) = [1 << x for x in range(24, 27)]


class V4l2_In_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DV_TIMINGS, STD, NATIVE_SIZE) = [1 << x for x in range(1, 4)]
    CUSTOM_TIMINGS = DV_TIMINGS


class V4l2_Output(Structure):
    _fields_ = [('index', c_uint32), ('name', c_uint8 * 32),
                ('type', c_uint32), ('audioset', c_uint32),
                ('modulator', c_uint32), ('std', V4l2_Std_Id),
                ('capabilities', c_uint32), ('reserved', c_uint32 * 3)]


class V4l2_Output_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (MODULATOR, ANALOG, ANALOGVGAOVERLAY) = range(1, 4)


class V4l2_Out_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DV_TIMINGS, STD, NATIVE_SIZE) = [1 << x for x in range(1, 4)]
    CUSTOM_TIMINGS = DV_TIMINGS


class V4l2_Control(Structure):
    _fields_ = [('id', c_uint32), ('value', c_int32)]


class V4l2_Ext_Control(Structure):
    class _u(Union):
        _fields_ = [('value', c_int32), ('value64', c_int64),
                    ('string', POINTER(c_char)), ('p_u8', POINTER(c_uint8)),
                    ('p_u16', POINTER(c_uint16)), ('p_u32', POINTER(c_uint32)),
                    ('p_area', POINTER(V4l2_Area)), ('ptr', POINTER(c_void_p))]

    _fields_ = [
        ('id', c_uint32),
        ('size', c_uint32),
        ('reserved2', c_uint32),
        ('_u', _u),
    ]
    _pack_ = True


class V4l2_Ext_Controls(Structure):
    class _u(Union):
        _fields_ = [('which', c_uint32)]

    _fields_ = [('_u', _u), ('count', c_uint32), ('error_idx', c_uint32),
                ('request_fd', c_uint32), ('reserved', c_uint32),
                ('controls', POINTER(V4l2_Ext_Control))]


def v4l2_ctrl_id2which(id):
    c = 0x0fff0000
    return id & c_ulong(c).value


def v4l2_ctrl_driver_priv(id):
    return (((id) & 0xffff) >= 0x1000)


class V4l2_Ctrl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ID_MASK = 0x0fffffff
    MAX_DIMS = 4
    WHICH_CUR_VAL = 0
    WHICH_DEF_VAL = 0x0f000000
    WHICH_REQUEST_VAL = 0x0f010000


class V4l2_Ctrl_Type(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (INTEGER, BOOLEAN, MENU, BUTTON, INTEGER64, CTRL_CLASS, STRING,
     BITMASK) = range(1, 9)
    COMPOUND_TYPES = 0x0100
    (U8, U16, U32) = [(1 << 8) + x for x in range(3)]
    AREA = 0x0106


class V4l2_Queryctrl(Structure):
    _fields_ = [('id', c_uint32), ('type', c_uint32), ('name', c_uint8 * 32),
                ('minimum', c_int32), ('maximum', c_int32), ('step', c_int32),
                ('default_value', c_int32), ('flags', c_uint32),
                ('reserved', c_uint32 * 2)]


class V4l2_Query_Ext_Ctrl(Structure):
    _fields_ = [('id', c_uint32), ('type', c_uint32), ('name', c_char * 32),
                ('minimum', c_int64), ('maximum', c_int64), ('step', c_uint64),
                ('default_value', c_int64), ('flags', c_uint32),
                ('elem_size', c_uint32), ('elems', c_uint32),
                ('nr_of_dims', c_uint32),
                ('dims', c_uint32 * V4l2_Ctrl.MAX_DIMS),
                ('reserved', c_uint32 * 32)]


class V4l2_Querymenu(Structure):
    class _u(Union):
        _fields_ = [('name', c_uint8 * 32), ('value', c_int64)]

    _fields_ = [('id', c_uint32), ('index', c_uint32), ('_u', _u),
                ('reserved', c_uint32)]
    _pack_ = True


class V4l2_Ctrl_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (DISABLED, GRABBED, READ_ONLY, UPDATE, INACTIVE, SLIDER, WRITE_ONLY,
     VOLATILE, HAS_PAYLOAD, EXECUTE_ON_WRITE,
     MODIFY_LAYOUT) = [1 << x for x in range(11)]

    NEXT_COMPOUND = 0x40000000
    NEXT_CTRL = 0x80000000


class V4l2_Cid(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    MAX_CTRLS = 0x400
    PRIVATE_BASE = 0x08000000


class V4l2_Tuner(Structure):
    _fields_ = [('index', c_uint32), ('name', c_uint8 * 32),
                ('type', c_uint32), ('capability', c_uint32),
                ('rangelow', c_uint32), ('rangehigh', c_uint32),
                ('rxsubchans', c_uint32), ('audmode', c_uint32),
                ('signal', c_int32), ('afc', c_int32),
                ('reserved', c_uint32 * 4)]


class V4l2_Modulator(Structure):
    _fields_ = [('index', c_uint32), ('name', c_uint8),
                ('capability', c_uint32), ('rangelow', c_uint32),
                ('rangehigh', c_uint32), ('txsubchans', c_uint32),
                ('type', c_uint32), ('reserved', c_uint32 * 3)]


class V4l2_Tuner_Cap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (LOW, NORM, HWSEEK_BOUNDED, HWSEEK_WRAP, STEREO, LANG2, LANG1, RDS,
     RDS_BLOCK_IO, RDS_CONTROLS, FREQ_BANDS, HWSEEK_PROG_LIM,
     T1HZ) = [1 << x for x in range(13)]
    SAP = (1 << 5)


class V4l2_Tuner_Sub(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (MONO, STEREO, LANG2, LANG1, RDS) = [1 << x for x in range(5)]
    SAP = (1 << 2)


class V4l2_Tuner_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (MONO, STEREO, LANG2, LANG1, LANG1_LANG2) = range(5)

    SAP = 0x0002


class V4l2_Frequency(Structure):
    _fields_ = [
        ('tuner', c_uint32),
        ('type', c_uint32),
        ('frecuency', c_uint32),
        ('reserved', c_uint32 * 8),
    ]


class V4l2_Band_Modulation(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (VSB, FM, AM) = [1 << x for x in range(3)]


class V4l2_Frequency_Band(Structure):
    _fields_ = [
        ('tuner', c_uint32),
        ('type', c_uint32),
        ('index', c_uint32),
        ('capability', c_uint32),
        ('rangelow', c_uint32),
        ('rangehigh', c_uint32),
        ('modulation', c_uint32),
        ('reserved', c_uint32 * 9),
    ]


class V4l2_Hw_Freq_Seek(Structure):
    _fields_ = [('tuner', c_uint32), ('type', c_uint32),
                ('seek_upward', c_uint32), ('wrap_around', c_uint32),
                ('spacing', c_uint32), ('rangelow', c_uint32),
                ('rangehigh', c_uint32), ('reserved', c_uint32 * 5)]


class V4l2_Rds_Data(Structure):
    _fields_ = [('lsb', c_uint8), ('msb', c_uint8), ('block', c_uint8)]
    _pack_ = True


class V4l2_Rds_Block(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    MSK = 0x7
    (A, B, C, D, C_ALT) = range(5)
    INVALID = 7
    (CORRECTED, ERROR) = [1 << x for x in range(6, 8)]


class V4l2_Audio(Structure):
    _fields_ = [('index', c_uint32), ('name', c_uint8),
                ('capability', c_uint32), ('mode', c_uint32),
                ('reserved', c_uint32 * 2)]


class V4l2_Audcap(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (STEREO, AVL) = [1 << x for x in range(2)]


class V4l2_Audmode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    AVL = 0x00001


class V4l2_Audioout(Structure):
    _fields_ = [('index', c_uint32), ('name', c_uint8),
                ('capability', c_uint32), ('mode', c_uint32),
                ('reserved', c_uint32 * 2)]


class V4l2_Enc_Idx_Frame(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (I, P, B) = range(3)
    MASK = (0xf)


class V4l2_Enc_Idx_Entry(Structure):
    _fields_ = [('offset', c_uint64), ('pts', c_uint64), ('length', c_uint32),
                ('flags', c_uint32), ('reserved', c_uint32 * 2)]


class V4l2_Enc_Idx(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    ENTRIES = 64


class V4l2_Enc_Idx(Structure):
    _fields_ = [('entries', c_uint32), ('entries_cap', c_uint32),
                ('reserved', c_uint32 * 4),
                ('entry', V4l2_Enc_Idx_Entry * V4l2_Enc_Idx.ENTRIES)]


class V4l2_Enc_Cmd(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (START, STOP, PAUSE, RESUME) = range(4)
    STOP_AT_GOP_END = (1 << 0)


class V4l2_Encoder_Cmd(Structure):
    class _u(Union):
        class _s(Structure):
            _fields_ = [
                ('data', c_uint32 * 8),
            ]

        _fields_ = [
            ('raw', _s),
        ]

    _fields_ = [('cmd', c_uint32), ('flags', c_uint32), ('_u', _u)]


class V4l2_Dec_Cmd(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (START, STOP, PAUSE, RESUME, FLUSH) = range(5)

    (START_MUTE_AUDIO, PAUSE_TO_BLACK, STOP_TO_BLACK) = [(1 << 0)] * 3
    STOP_IMMEDIATELY = (1 << 1)


class V4l2_Dec_Start_Fmt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    NONE = 0
    GOP = 1


class V4l2_Decoder_Cmd(Structure):
    class _u(Union):
        class _stop(Structure):
            _fields_ = [
                ('pts', c_uint64),
            ]

        class _start(Structure):
            _fields_ = [
                ('speed', c_int32),
                ('format', c_uint32),
            ]

        class _raw(Structure):
            _fields_ = [
                ('data', c_uint32 * 16),
            ]

        _fields_ = [('stop', _stop), ('start', _start), ('raw', _raw)]

    _fields_ = [('cmd', c_uint32), ('flags', c_uint32), ('_u', _u)]


class V4l2_Vbi(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    UNSYNC = (1 << 0)
    INTERLACED = (1 << 1)
    ITU_525_F1_START = (1)
    ITU_525_F2_START = (264)
    ITU_625_F1_START = (1)
    ITU_625_F2_START = (314)


class V4l2_Vbi_Format(Structure):
    _fields_ = [('sampling_rate', c_uint32), ('offset', c_uint32),
                ('samples_per_line', c_uint32), ('sample_format', c_uint32),
                ('start', c_int32 * 2), ('count', c_uint32 * 2),
                ('flags', c_uint32), ('reserved', c_uint32 * 2)]


class V4l2_Sliced_Vbi_Format(Structure):
    _fields_ = [('service_set', c_uint16),
                ('service_lines', c_uint16 * 2 * 24), ('io_size', c_uint32),
                ('reserved', c_uint32 * 2)]


class V4l2_Sliced(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    TELETEXT_B = 0x0001
    VPS = 0x0400
    CAPTION_525 = 0x1000
    WSS_625 = 0x4000
    VBI_525 = CAPTION_525
    VBI_625 = (TELETEXT_B | VPS | WSS_625)


class V4l2_Sliced_Vbi_Cap(Structure):
    _fields_ = [('service_set', c_uint16),
                ('service_lines', c_uint16 * 2 * 24), ('type', c_uint32),
                ('reserved', c_uint32 * 3)]


class V4l2_Sliced_Vbi_Data(Structure):
    _fields_ = [
        ('id', c_uint32),
        ('field', c_uint32),
        ('line', c_uint32),
        ('reserved', c_uint32),
        ('data', c_uint8 * 48),
    ]


class V4l2_Mpeg_Vbi_Ivtv(IntEnum):
    def __c_type__():
        return c_uint8

    def __str__(self):
        return '{0}'.format(self.value)

    TELETEXT_B = 1
    CAPTION_525 = 4
    WSS_625 = 5
    VPS = 7


class V4l2_Mpeg_Vbi_Ivtv_Magic(Enum):
    def __str__(self):
        return '{0}'.format(self.value)

    M0 = "itv0"
    M1 = "ITV0"


class V4l2_Mpeg_Vbi_Itv0_Line(Structure):
    _fields_ = [('id', c_uint8), ('data', c_uint8 * 42)]
    _pack_ = True


class V4l2_Mpeg_Vbi_Itv0(Structure):
    class _s(LittleEndianStructure):
        _fields_ = [('linemask', c_uint32)]

    _fields_ = [('_s', _s * 2), ('line', V4l2_Mpeg_Vbi_Itv0_Line * 35)]
    _pack_ = True


class V4l2_Mpeg_Vbi_ITV0(Structure):
    _fields_ = [('line', V4l2_Mpeg_Vbi_Itv0_Line * 35)]
    _pack_ = True


class V4l2_Mpeg_Vbi_Fmt_Ivtv(Structure):
    class _u(Union):
        _fields_ = [('itv0', V4l2_Mpeg_Vbi_Itv0), ('ITV0', V4l2_Mpeg_Vbi_ITV0)]

    _fields_ = [('magic', c_uint8 * 4), ('_u', _u)]
    _pack_ = True


class V4l2_Plane_Pix_Format(Structure):
    _fields_ = [('sizeimage', c_uint32), ('bytesperline', c_uint32),
                ('reserved', c_uint16 * 6)]
    _pack_ = True


class V4l2_Pix_Format_Mplane(Structure):
    class _u(Union):
        _fields_ = [('ycbcr_enc', c_uint8), ('hsv_enc', c_uint8)]

    _fields_ = [('width', c_uint32), ('height', c_uint32),
                ('pixelformat', c_uint32), ('field', c_uint32),
                ('colorspace', c_uint32),
                ('plane_fmt',
                 V4l2_Plane_Pix_Format * Video_Max.PLANES),
                ('num_planes', c_uint8), ('flags', c_uint8), ('_u', _u),
                ('quantization', c_uint8), ('xfer_func', c_uint8),
                ('reserved', c_uint8 * 7)]

    _pack_ = True


class V4l2_Sdr_Format(Structure):
    _fields_ = [('pixelformat', c_uint32), ('buffersize', c_uint32),
                ('reserved', c_uint8 * 24)]
    _pack_ = True


class V4l2_Meta_Format(Structure):
    _fields_ = [('dataformat', c_uint32), ('buffersize', c_uint32)]
    _pack_ = True


class V4l2_Format(Structure):
    class _fmt(Union):
        _fields_ = [
            ('pix', V4l2_Pix_Format), ('pix_mp', V4l2_Pix_Format_Mplane),
            ('win', V4l2_Window), ('vbi', V4l2_Vbi_Format),
            ('sliced', V4l2_Sliced_Vbi_Format), ('sdr', V4l2_Sdr_Format),
            ('meta', V4l2_Meta_Format), ('raw_data', c_uint8 * 200)
        ]

    _fields_ = [('type', c_uint32), ('fmt', _fmt)]


class V4l2_Streamparm(Structure):
    class _parm(Union):
        _fields_ = [('capture', V4l2_Captureparm), ('output', V4l2_Outputparm),
                    ('raw_data', c_uint8 * 200)]

    _fields_ = [('type', c_uint32), ('parm', _parm)]


class V4l2_Event(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (ALL, VSYNC, EOS, CTRL, FRAME_SYNC, SOURCE_CHANGE, MOTION_DET) = range(7)

    PRIVATE_START = 0x08000000


class V4l2_Event_Vsync(Structure):
    _fields_ = [('field', c_uint32)]
    _pack_ = True


class V4l2_Event_Ctrl_Ch(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (VALUE, FLAGS, RANGE) = [1 << x for x in range(3)]


class V4l2_Event_Ctrl(Structure):
    class _u(Union):
        _fields_ = [('value', c_int32), ('value64', c_int64)]

    _fields_ = [('changes', c_uint32), ('type', c_uint32), ('_u', _u),
                ('flags', c_uint32), ('minimum', c_int32),
                ('maximum', c_int32), ('step', c_int32),
                ('default_value', c_int32)]


class V4l2_Event_Frame_Sync(Structure):
    _fields_ = [('frame_sequence', c_uint32)]


class V4l2_Event_Src_Ch(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    RESOLUTION = (1 << 0)


class V4l2_Event_Src_Change(Structure):
    _fields_ = [('changes', c_uint32)]


class V4l2_Event_Md_Fl_Have(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    FRAME_SEQ = (1 << 0)


class V4l2_Event_Motion_Det(Structure):
    _fields_ = [('flags', c_uint32), ('frame_sequence', c_uint32),
                ('region_mask', c_uint32)]


class V4l2_Event(Structure):
    class timespec(Structure):
        _fields_ = [('tv_sec', c_long), ('tv_nsec', c_long)]

    class _u(Union):
        _fields_ = [('vsync', V4l2_Event_Vsync), ('ctrl', V4l2_Event_Ctrl),
                    ('frame_sync', V4l2_Event_Frame_Sync),
                    ('src_change', V4l2_Event_Src_Change),
                    ('motion_det', V4l2_Event_Motion_Det),
                    ('data', c_uint8 * 64)]

    _fields_ = [('type', c_uint32), ('u', _u), ('pending', c_uint32),
                ('sequence', c_uint32), ('timestamp', timespec),
                ('id', c_uint32), ('reserved', c_uint32 * 8)]


class V4l2_Event_Sub_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (SEND_INITIAL, ALLOW_FEEDBACK) = [1 << x for x in range(2)]


class V4l2_Event_Subscription(Structure):
    _fields_ = [
        ('type', c_uint32),
        ('id', c_uint32),
        ('flags', c_uint32),
        ('reserved', c_uint32 * 5),
    ]


class V4l2_Chip_Match(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (BRIDGE, I2C_DRIVER, I2C_ADDR, AC97, SUBDEV) = range(5)
    HOST = BRIDGE


class V4l2_Dbg_Match(Structure):
    class _u(Union):
        _fields_ = [('addr', c_uint32), ('name', c_char * 32)]

    _fields_ = [('type', c_uint32), ('_u', _u)]
    _pack_ = True


class V4l2_Dbg_Register(Structure):
    _fields_ = [('match', V4l2_Dbg_Match), ('size', c_uint32),
                ('reg', c_uint64), ('val', c_uint64)]
    _pack_ = True


class V4l2_Chip_Fl(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (READABLE, WRITABLE) = range(2)


class V4l2_Dbg_Chip_Info(Structure):
    _fields_ = [('match', V4l2_Dbg_Match), ('name', c_char * 32),
                ('flags', c_uint32), ('reserved', c_uint32 * 32)]
    _pack_ = True


class V4l2_Create_Buffers(Structure):
    _fields_ = [('index', c_uint32), ('count', c_uint32), ('memory', c_uint32),
                ('format', V4l2_Format), ('capabilities', c_uint32),
                ('reserved', c_uint32 * 7)]


class Vidioc(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    QUERYCAP = ior('V', 0, V4l2_Capability)
    ENUM_FMT = iow('V', 2, V4l2_Fmtdesc)
    G_FMT = iowr('V', 4, V4l2_Format)
    S_FMT = iowr('V', 5, V4l2_Format)
    REQBUFS = iowr('V', 8, V4l2_Requestbuffers)
    QUERYBUF = iowr('V', 9, V4l2_Buffer)
    G_FBUF = ior('V', 10, V4l2_Framebuffer)
    S_FBUF = iow('V', 11, V4l2_Framebuffer)
    OVERLAY = iow('V', 14, c_int)
    QBUF = iowr('V', 15, V4l2_Buffer)
    EXPBUF = iowr('V', 16, V4l2_Exportbuffer)
    DQBUF = iowr('V', 17, V4l2_Buffer)
    STREAMON = iow('V', 18, c_int)
    STREAMOFF = iow('V', 19, c_int)
    G_PARM = iowr('V', 21, V4l2_Streamparm)
    S_PARM = iowr('V', 22, V4l2_Streamparm)
    G_STD = ior('V', 23, V4l2_Std_Id)
    S_STD = iow('V', 24, V4l2_Std_Id)
    ENUMSTD = iowr('V', 25, V4l2_Standard)
    ENUMINPUT = iowr('V', 26, V4l2_Input)
    G_CTRL = iowr('V', 27, V4l2_Control)
    S_CTRL = iowr('V', 28, V4l2_Control)
    G_TUNER = iowr('V', 29, V4l2_Tuner)
    S_TUNER = iow('V', 30, V4l2_Tuner)
    G_AUDIO = ior('V', 33, V4l2_Audio)
    S_AUDIO = iow('V', 34, V4l2_Audio)
    QUERYCTRL = iowr('V', 36, V4l2_Queryctrl)
    QUERYMENU = iowr('V', 37, V4l2_Querymenu)
    G_INPUT = ior('V', 38, c_int)
    S_INPUT = iowr('V', 39, c_int)
    G_EDID = iowr('V', 40, V4l2_Edid)
    S_EDID = iowr('V', 41, V4l2_Edid)
    G_OUTPUT = ior('V', 46, c_int)
    S_OUTPUT = iowr('V', 47, c_int)
    ENUMOUTPUT = iowr('V', 48, V4l2_Output)
    G_AUDOUT = ior('V', 49, V4l2_Audioout)
    S_AUDOUT = iow('V', 50, V4l2_Audioout)
    G_MODULATOR = iowr('V', 54, V4l2_Modulator)
    S_MODULATOR = iow('V', 55, V4l2_Modulator)
    G_FREQUENCY = iowr('V', 56, V4l2_Frequency)
    S_FREQUENCY = iow('V', 57, V4l2_Frequency)
    CROPCAP = iowr('V', 58, V4l2_Cropcap)
    G_CROP = iowr('V', 59, V4l2_Crop)
    S_CROP = iow('V', 60, V4l2_Crop)
    G_JPEGCOMP = ior('V', 61, V4l2_Jpegcompression)
    S_JPEGCOMP = iow('V', 62, V4l2_Jpegcompression)
    QUERYSTD = ior('V', 63, V4l2_Std_Id)
    TRY_FMT = iowr('V', 64, V4l2_Format)
    ENUMAUDIO = iowr('V', 65, V4l2_Audio)
    ENUMAUDOUT = iowr('V', 66, V4l2_Audioout)
    G_PRIORITY = ior('V', 67, c_uint32)
    S_PRIORITY = iow('V', 68, c_uint32)
    G_SLICED_VBI_CAP = iowr('V', 69, V4l2_Sliced_Vbi_Cap)
    LOG_STATUS = io('V', 70)
    G_EXT_CTRLS = iowr('V', 71, V4l2_Ext_Controls)
    S_EXT_CTRLS = iowr('V', 72, V4l2_Ext_Controls)
    TRY_EXT_CTRLS = iowr('V', 73, V4l2_Ext_Controls)
    ENUM_FRAMESIZES = iowr('V', 74, V4l2_Frmsizeenum)
    ENUM_FRAMEINTERVALS = iowr('V', 75, V4l2_Frmivalenum)
    G_ENC_INDEX = ior('V', 76, V4l2_Enc_Idx)
    ENCODER_CMD = iowr('V', 77, V4l2_Encoder_Cmd)
    TRY_ENCODER_CMD = iowr('V', 78, V4l2_Encoder_Cmd)
    DBG_S_REGISTER = iow('V', 79, V4l2_Dbg_Register)
    DBG_G_REGISTER = iowr('V', 80, V4l2_Dbg_Register)
    S_HW_FREQ_SEEK = iow('V', 82, V4l2_Hw_Freq_Seek)
    S_DV_TIMINGS = iowr('V', 87, V4l2_Dv_Timings)
    G_DV_TIMINGS = iowr('V', 88, V4l2_Dv_Timings)
    DQEVENT = ior('V', 89, V4l2_Event)
    SUBSCRIBE_EVENT = iow('V', 90, V4l2_Event_Subscription)
    UNSUBSCRIBE_EVENT = iow('V', 91, V4l2_Event_Subscription)
    CREATE_BUFS = iowr('V', 92, V4l2_Create_Buffers)
    PREPARE_BUF = iowr('V', 93, V4l2_Buffer)
    G_SELECTION = iowr('V', 94, V4l2_Selection)
    S_SELECTION = iowr('V', 95, V4l2_Selection)
    DECODER_CMD = iowr('V', 96, V4l2_Decoder_Cmd)
    TRY_DECODER_CMD = iowr('V', 97, V4l2_Decoder_Cmd)
    ENUM_DV_TIMINGS = iowr('V', 98, V4l2_Enum_Dv_Timings)
    QUERY_DV_TIMINGS = ior('V', 99, V4l2_Dv_Timings)
    DV_TIMINGS_CAP = iowr('V', 100, V4l2_Dv_Timings_Cap)
    ENUM_FREQ_BANDS = iowr('V', 101, V4l2_Frequency_Band)
    DBG_G_CHIP_INFO = iowr('V', 102, V4l2_Dbg_Chip_Info)
    QUERY_EXT_CTRL = iowr('V', 103, V4l2_Query_Ext_Ctrl)


class BASE_VIDIOC(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    PRIVATE = 192


from ctypes import *
from enum import Enum, IntEnum
from util.ioctl import *
from util.ctypes_tools import *
from v4l2.common import *


class Pulse(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    BIT = 0x01000000
    MASK = 0x00FFFFFF


class Lirc_Mode2(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    SPACE = 0x00000000
    PULSE = 0x01000000
    FREQUENCY = 0x02000000
    TIMEOUT = 0x03000000
    MASK = 0xFF000000


class Lirc_Value(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    MASK = 0x00FFFFFF


def lirc_space(val):
    return (((val) & Lirc_Value.MASK) | Lirc_Mode2.SPACE)


def lirc_pulse(val):
    return (((val) & Lirc_Value.MASK) | Lirc_Mode2.PULSE)


def lirc_frequency(val):
    return (((val) & Lirc_Value.MASK) | Lirc_Mode2.FREQUENCY)


def lirc_timeout(val):
    return (((val) & Lirc_Value.MASK) | Lirc_Mode2.TIMEOUT)


def lirc_value(val):
    return ((val) & Lirc_Value.MASK)


def lirc_mode2(val):
    return ((val) & Lirc_Mode2.MASK)


def lirc_is_space(val):
    return (lirc_mode2(val) == Lirc_Mode2.SPACE)


def lirc_is_pulse(val):
    return (lirc_mode2(val) == Lirc_Mode2.PULSE)


def lirc_is_frequency(val):
    return (lirc_mode2(val) == Lirc_Mode2.FREQUENCY)


def lirc_is_timeout(val):
    return (lirc_mode2(val) == Lirc_Mode2.TIMEOUT)


lirc_t = int


def lirc_mode2send(x):
    return (x)


def lirc_send2mode(x):
    return (x)


def lirc_mode2rec(x):
    return ((x) << 16)


def lirc_rec2mode(x):
    return ((x) >> 16)


class Lirc_Mode(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (RAW, PULSE, MODE2, SCANCODE, LIRCCODE) = [1 << x for x in range(5)]


class Lirc_Can(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    SEND_RAW = lirc_mode2send(Lirc_Mode.RAW)
    SEND_PULSE = lirc_mode2send(Lirc_Mode.PULSE)
    SEND_MODE2 = lirc_mode2send(Lirc_Mode.MODE2)
    SEND_LIRCCODE = lirc_mode2send(Lirc_Mode.LIRCCODE)
    SEND_MASK = 0x0000003f
    SET_SEND_CARRIER = 0x00000100
    SET_SEND_DUTY_CYCLE = 0x00000200
    SET_TRANSMITTER_MASK = 0x00000400
    REC_RAW = lirc_mode2rec(Lirc_Mode.RAW)
    REC_PULSE = lirc_mode2rec(Lirc_Mode.PULSE)
    REC_MODE2 = lirc_mode2rec(Lirc_Mode.MODE2)
    REC_SCANCODE = lirc_mode2rec(Lirc_Mode.SCANCODE)
    REC_LIRCCODE = lirc_mode2rec(Lirc_Mode.LIRCCODE)
    REC_MASK = lirc_mode2rec(SEND_MASK)
    SET_REC_CARRIER = (SET_SEND_CARRIER << 16)
    SET_REC_DUTY_CYCLE = (SET_SEND_DUTY_CYCLE << 16)
    SET_REC_DUTY_CYCLE_RANGE = 0x40000000
    SET_REC_CARRIER_RANGE = 0x80000000
    GET_REC_RESOLUTION = 0x20000000
    SET_REC_TIMEOUT = 0x10000000
    SET_REC_FILTER = 0x08000000
    MEASURE_CARRIER = 0x02000000
    USE_WIDEBAND_RECEIVER = 0x04000000
    NOTIFY_DECODE = 0x01000000


def lirc_can_send(x):
    return ((x) & Lirc_Can.SEND_MASK)


def lirc_can_rec(x):
    return ((x) & Lirc_Can.REC_MASK)


class Lirc(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    GET_FEATURES = ior('i', 0x00000000, c_uint32)
    GET_SEND_MODE = ior('i', 0x00000001, c_uint32)
    GET_REC_MODE = ior('i', 0x00000002, c_uint32)
    GET_REC_RESOLUTION = ior('i', 0x00000007, c_uint32)
    GET_MIN_TIMEOUT = ior('i', 0x00000008, c_uint32)
    GET_MAX_TIMEOUT = ior('i', 0x00000009, c_uint32)
    GET_LENGTH = ior('i', 0x0000000f, c_uint32)
    SET_SEND_MODE = iow('i', 0x00000011, c_uint32)
    SET_REC_MODE = iow('i', 0x00000012, c_uint32)
    SET_SEND_CARRIER = iow('i', 0x00000013, c_uint32)
    SET_REC_CARRIER = iow('i', 0x00000014, c_uint32)
    SET_SEND_DUTY_CYCLE = iow('i', 0x00000015, c_uint32)
    SET_TRANSMITTER_MASK = iow('i', 0x00000017, c_uint32)
    SET_REC_TIMEOUT = iow('i', 0x00000018, c_uint32)
    SET_REC_TIMEOUT_REPORTS = iow('i', 0x00000019, c_uint32)
    SET_MEASURE_CARRIER_MODE = iow('i', 0x0000001d, c_uint32)
    SET_REC_CARRIER_RANGE = iow('i', 0x0000001f, c_uint32)
    SET_WIDEBAND_RECEIVER = iow('i', 0x00000023, c_uint32)
    GET_REC_TIMEOUT = ior('i', 0x00000024, c_uint32)


class Lirc_Scancode(Structure):
    _fields_ = [('timestamp', c_uint64), ('flags', c_uint16),
                ('rc_proto', c_uint16), ('keycode', c_uint32),
                ('scancode', c_uint64)]


class Lirc_Scancode_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (TOGGLE, REPEAT) = range(1, 3)


class Rc_Proto(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (UNKNOWN, OTHER, RC5, RC5X_20, RC5_SZ, JVC, SONY12, SONY15, SONY20, NEC,
     NECX, NEC32, SANYO, MCIR2_KBD, MCIR2_MSE, RC6_0, RC6_6A_20, RC6_6A_24,
     RC6_6A_32, RC6_MCE, SHARP, XMP, CEC, IMON, RCMM12, RCMM24, RCMM32,
     XBOX_DVD) = range(28)



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='uapy',
    version='1.0.0',
    author="Andres Castillo",
    author_email="indigohedgehog@gmail.com",
    description='Python wrapper for Linux UAPI ioctl',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/indigohedgehog/uapy",
    project_urls={
        "Bug Tracker": "https://github.com/indigohedgehog/uapy/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX :: Linux", "Topic :: Multimedia",
        "Topic :: Multimedia :: Video"
    ],
    install_requires=[
        'ctypes',
    ],    
    package_dir={"": "uapy"},
    packages=find_packages(where="uapy"),
    python_requires=">=3.6",
)


from ctypes import *
from enum import IntEnum


class V4l2_Sel_Tgt(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    CROP = 0x0000

    (CROP_DEFAULT, CROP_BOUNDS, NATIVE_SIZE) = [(1 << 0) + x for x in range(3)]

    (COMPOSE, COMPOSE_DEFAULT, COMPOSE_BOUNDS,
     COMPOSE_PADDED) = [(1 << 8) + x for x in range(4)]


class V4l2_Sel_Flag(IntEnum):
    def __str__(self):
        return '{0}'.format(self.value)

    (GE, LE, KEEP_CONFIG) = [1 << x for x in range(3)]


class V4l2_Edid(Structure):
    _fields_ = [('pad', c_uint32), ('start_block', c_uint32),
                ('blocks', c_uint32), ('reserved', c_uint32 * 5),
                ('edid', POINTER(c_uint8))]
